#include <formats/CCFormat.h>
#include <ccbg/CCMol.h>

class FASTAFormat : public CCFormat
{
public:
    FASTAFormat()
    {
        RegisterFormat( "fasta", this );
        RegisterFormat( "fa", this );
        RegisterFormat( "fsa", this );
    }
    virtual ~FASTAFormat()
    {
    }

	bool ReadMolecule( CCMol& mol, istream& is ) 
	{
		cout << "write only type\n";
		return false;
	}
	bool ReadAtomsOnly( CCMol& mol, istream& is ) {
		return ReadMolecule( mol, is );
	}
    bool WriteMolecule( CCMol& mol, ostream& os );
    virtual const char* Description()
    {
        return
        "FASTA format\n" "http://www.ebi.ac.uk/help/formats_frmae.html#fasta";
    };
};
FASTAFormat theFASTAFormat;

bool FASTAFormat::WriteMolecule( CCMol& mol, ostream& os )
{
	if ( mol.IsEmpty() ) {
		gErrorLog.Throw( __FUNCTION__, "Molecule is empty." );
		return false;
	}

    string seq;
    FOR_RESIDUES_OF_MOL( r, mol ) {
        seq += r->OneLetter();
    }

	string molName = mol.GetName();
    if ( !molName.empty() ) {
		// set max length of molecular name as 50 characters
        os << '>' << molName.substr( 0, 50 );
    }
    else {
        os << "> UNK MOL";
    }
    os << " " << mol.GetNumRes() << " bp\n";
    os << seq << endl;
    return true;
}

