#ifndef CCCHAIN_H
#define CCCHAIN_H

#include <CCConfig.h>
#include <ccbg/CCBase.h>

class CCAtom;
class CCBond;
class CCMol;
class CCResidue;
class CCChain;

typedef std::vector<CCChain*>::iterator CCChainIter;

/************************************************************************/
/*                           CCChain class                              */
/************************************************************************/

class CCAPI CCChain : public CCBase
{
protected:
    CCMol* m_mol;      // Parent molecule
    int m_idx;      // Chain index
    char m_chnid;    // Chain ID
    vector<CCResidue*> m_vresidue; // Residues

public:
    CCChain();
    virtual ~CCChain();
    CCChain( const CCChain& src );
    CCChain& operator=( const CCChain& src );

    CCMol* GetMol() const
    {
        return m_mol;
    }
    int GetIdx() const
    {
        return m_idx;
    }
    int GetChainID() const
    {
        return m_chnid;
    }
    CCResidue* GetResidue( int idx ) const;
    vector<CCResidue*>& GetResidues()
    {
        return m_vresidue;
    }
    int GetNumRes() const
    {
        return m_vresidue.size();
    }
    void SetMol( CCMol* mol )
    {
        m_mol = mol;
    }
    void SetIdx( int idx )
    {
        m_idx = idx;
    }
    void SetChainID( char chain )
    {
        m_chnid = chain;
    }

    CCResidue* BeginResidue( vector<CCResidue*>::iterator& i );
    CCResidue* NextResidue( vector<CCResidue*>::iterator& i );
    void AddResidue( CCResidue* res );
    void RemoveResidue( CCResidue* res );
};
/************************************************************************/
/* Chain related functions                                              */
/************************************************************************/

#endif

