/** 
* @file 
* @brief interface for handling error
* @author Yuan ZHAO, Tiejun CHENG
* @date 2008/09/09
* @version 1.0.1 
*
*/

#ifndef CCERROR_H
#define CCERROR_H

#include "CCConfig.h"

enum ccMsgLevel
{
    ccUnknown,    ///< unknown error
    ccDebug,      ///< for debug only
    ccWarning,    ///< non-critical warning
    ccError,      ///< error
    ccFatal,      ///< fatal error
};

/**
* @class CCErrorLog CCError.h "CCError.h"
* @brief interface for error throwing
*/
class CCAPI CCErrorLog
{
	/**
	* @class CCError CCError.h "include/ccbg/CCError.h"
	* @brief interface for error message
	*/
	class CCAPI CCError
	{
	public:
		/** @brief constructor */
		CCError( const string& method = "", const string& errorMsg = "",
			const ccMsgLevel level = ccWarning )
			: m_method( method ), m_errorMsg( errorMsg ), m_level( level )
		{
		}
		/** @brief destructor */
		virtual ~CCError() {};
		/** @brief format and output an error message */
		string message() const;
		/** @brief reload operator << */
		friend ostream& operator <<( ostream& os, const CCError& errorMsg )
		{
			return os << errorMsg.message() << endl;
		}

	protected:
		string m_errorMsg;  ///< error message string
		string m_method;    ///< function throwing an error
		ccMsgLevel m_level; ///< error message level
	};

public:
	/** @brief constructor */
    CCErrorLog( ostream* oss = &clog, const string& logFile = "" );
	/** @brief destructor */
    virtual ~CCErrorLog();

	/** @brief throw an error */
    void Throw( CCError& error );
	/** @brief throw an error */
    void Throw( const string& method, const string& errorMsg,
                ccMsgLevel level = ccWarning );

protected:
    ostream* m_oss; ///< destination to which an error goes, typically std::clog
    ostream* m_ofs; ///< second destination, can be used to throw error to file
};
/** @brief declaration for a global error handler */
EXTERN CCErrorLog gErrorLog;

#endif
