/** 
* @file 
* @brief interface for handling file under WIN32 and UNIX/Linux
* @author Yuan ZHAO, Tiejun CHENG
* @date 2008/09/09
* @version 1.0.1 
*
* the following codes print all files that match pattern 'C:\\*.mol2'
* @code
* vector<string> flist;
* vector<string>::iterator it;
* FindFiles(flist, "C:\\*.mol2");
* for ( it=flist.begin(); it!=flist.end(); ++it ) {
*     cout << "Found file: " << *it << "\n";
* }
* @endcode
*
* Copyright (C) 2007-2008 www.sioc-ccbg.ac.cn
* 
* This file is part of the CCBGKit project.
*/

#ifndef CCFILEHANDLER_H
#define CCFILEHANDLER_H

#include <CCConfig.h>

/** @brief find files with given name */
CCAPI int FindFiles( vector<string>& file_list, const string& filename );
/** @brief find files with given pattern in directory */
CCAPI int FindFilesInDir( vector<string>& file_list, const string& path,
                          const string& pattern );
/** @brief find files with given pattern in directory */
CCAPI int FindFilesInDir( vector<string>& file_list, const string& path,
                          const string& pattern, int MAX_DEPTH );

/** @brief assert whether file opened successfully (C style) */
CCAPI void AssertOpenFile( FILE* fp, const string& filename );
/** @brief assert whether file opened successfully (C++ style) */
CCAPI void AssertOpenFile( ios& fs, const string& filename );

/** @brief parse full path into pieces */
CCAPI void FileParse( const string& fullpath, vector<string>& dfbe );
/** @brief return directory name (with '/' or '\\') of a full path */
CCAPI string DirName( const string& fullpath );
/** @brief return full file name (i.e. with extension) of a full path */
CCAPI string FileName( const string& fullpath );
/** @brief return base file name (i.e. without extension) of a full path */
CCAPI string BaseName( const string& fullpath );
/** @brief return extension name (without '.') of a full path  */
CCAPI string ExtName( const string& fullpaht );

#endif

