//

#include <ccbg/CCMol.h>
#include <string>
#include <vector>

// string type to double
double string2double(string s)
{
         stringstream ss; ss<<s; double d; ss>>d;
         return d;
}

// Protein atom type definition
int get_type(string res, string atm)
{
    if(atm=="C"||((res=="ASP"||res=="ASN")&&atm=="CG")||((res=="GLU"||res=="GLN")&&atm=="CD"))   // C.2 atom
    return 1;
    if(res=="ARG"&&atm=="CZ")              // C.cat
    return 2;
    if((res=="PHE"&&(atm=="CD1"||atm=="CD2"||atm=="CE1"||atm=="CE2"||atm=="CZ"||atm=="CG"))||(res=="TYR"&&(atm=="CG"||atm=="CD1"||atm=="CD2"||atm=="CE1"||atm=="CE2"||atm=="CZ"))||(res=="TRP"&&(atm=="CG"||atm=="CD1"||atm=="CD2"||atm=="CZ2"||atm=="CZ3"||atm=="CH2"||atm=="CE2"||atm=="CE3"))||(res=="HIS"&&(atm=="CD2"||atm=="CG"||atm=="CE1")))
    return 3;                              // C.ar atom 
    if((atm=="CA")||(atm=="CB"&&(res=="ALA"||res=="SER"||res=="THR"||res=="CYS"||res=="LEU"||res=="ILE"||res=="VAL"||res=="MET"||res=="ASP"||res=="GLU"||res=="ASN"||res=="GLN"||res=="PHE"||res=="TRP"||res=="TYR"||res=="HIS"||res=="LYS"||res=="ARG"||res=="PRO"))||(atm=="CG"&&(res=="LEU"||res=="GLU"||res=="GLN"||res=="LYS"||res=="ARG"||res=="PRO"))||(atm=="CG1"&&res=="ILE")||(atm=="CD"&&res=="LYS")||(res=="MET"&&atm=="CG")||(res=="VAL"&&(atm=="CG1"||atm=="CG2"))||(res=="LEU"&&(atm=="CD1"||atm=="CD2"))||(res=="ILE"&&(atm=="CD1"||atm=="CG2"))||(res=="THR"&&atm=="CG2")||(res=="MET"&&atm=="CE")||(res=="LYS"&&atm=="CE")||(res=="ARG"&&atm=="CD")||(res=="PRO"&&atm=="CD"))
    return 4;                             // C.3 atom 
    if(atm=="N"||(res=="ASN"&&atm=="ND2")||(res=="GLN"&&atm=="NE2"))               // N.am atom
    return 5;
    if(res=="ARG"&&(atm=="NH1"||atm=="NH2"||atm=="NE"))   // N.pl3 in ARG
    return 6;
    if((res=="TRP"&&atm=="NE1")||(res=="HIS"&&(atm=="ND1"||atm=="NE2")))             // N.ar
    return 7;
    if(res=="LYS"&&atm=="NZ")              // N.4 in LYS
    return 8;
    if(atm=="O"||(res=="ASN"&&atm=="OD1")||(res=="GLN"&&atm=="OE1"))            // O.2 
    return 9;
    if((res=="ASP"&&(atm=="OD1"||atm=="OD2"))||(res=="GLU"&&(atm=="OE1"||atm=="OE2")))
    return 10;                             //O.co2
    if((res=="SER"&&atm=="OG")||(res=="THR"&&atm=="OG1")||(res=="TYR"&&atm=="OH"))
    return 11;                             // O.3
    if(atm=="SD"||atm=="SG")               // S atom
    return 12;    
    else                                   // other atoms outside above definitions
    return 0;
}


// Ligand atom type definition
int get_type2(string p)
{
    if(p=="C.3")                           //  C atom, sp3           
    return 1;
    if(p=="C.2"||p=="C.CO2"||p=="C.Ng+")   //  C atom, sp2
    return 2;
    if(p=="C.ar")                          //  C atom, sp2, aromatic
    return 3;
    if(p=="N.3"||p=="N.4+")                //  N atom, sp3
    return 4;
    if(p=="N.2"||p=="N.ar")                //  N atom, sp2
    return 5;
    if(p=="N.am"||p=="N.Pam"||p=="N.Sam")  //  N atom linking with C=O, -S=O, -P=O group
    return 6;
    if(p=="N.pl3"||p=="N.2+"||p=="N.NOx"||p=="N.Ng+"||p=="N.NO"||p=="N.NO2")   //  N atom, sp3, planar
    return 7;
    if(p=="O.3"||p=="O.ar")                // O atom, sp3 and oxygen in aromatic ring
    return 8;
    if(p=="O.2"||p=="O.NO"||p=="O.NO2"||p=="O.NOx"||p=="O.PO"||p=="O.SO"||p=="O.SO2"||p=="O.CO2"||p=="O.PO2"||p=="O.PO3"||p=="O.SO3") // O atom, sp2
    return 9;
    if(p=="S.3"||p=="S.ar"||p=="S")        // S atom
    return 10;
    if(p=="F")                             // F atom
    return 11;
    if(p=="Cl"||p=="Br"||p=="I")           // Cl,Br,I
    return 12;
    else                                   // other atoms outside above definitions
    return 0;
}

// calculate pmf energies for water molecules interacted with ligand atoms
double pmf_lig(CCMol& m1, double x, double y, double z, vector<double> pmf_l, double c1, double c2)
{
     double score=0.0;
     double x1(0.0), y1(0.0), z1(0.0);
     int n=int((c2-c1)/0.1)+1;
     FOR_ATOMS_OF_MOL(a1,m1) {
          double d1(0.0);
          if (a1->IsHydrogen()) {  continue;  }
          else
          {
              x1=a1->GetX();
              y1=a1->GetY();
              z1=a1->GetZ();
              d1=sqrt((x1-x)*(x1-x)+(y1-y)*(y1-y)+(z1-z)*(z1-z));
              if(d1>c2)   { continue; }
              if(d1<c1)  { score+=4.0; }
              if(d1>=c1&&d1<=c2)
              {
                 int t1=0;
                 t1=get_type2(a1->GetType());
                 if(t1==0) { continue; }
                 if(t1>0)
                 {
                    int s0=int((d1-c1)/0.1);      
                    score+=pmf_l[n*(t1-1)+s0];
                 }
              }
          }
    }
    return score;
}

// calculate pmf energies for water molecules interacted with protein atoms
double pmf_pro(CCMol& m1, double x2, double y2, double z2, vector<double> pmf_p, double c1, double c2)
{
     double score2=0.0;
     double x1(0.0), y1(0.0), z1(0.0);
     int n=int((c2-c1)/0.1)+1;
     FOR_ATOMS_OF_MOL(a1,m1) {
          double d2(0.0);
          if (a1->IsHydrogen()) {  continue;  }
          else
          {
              x1=a1->GetX();
              y1=a1->GetY();
              z1=a1->GetZ();
              d2=sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2));
              if(d2>c2)   { continue; }
              if(d2<c1)  { score2+=4.0; }
              if(d2>=c1&&d2<=c2)
              {
                 int t2=0;
                 t2=get_type(a1->GetResName(), a1->GetName());
                 if(t2==0) { continue; }
                 if(t2>0)
                 {
                    int s0=int((d2-c1)/0.1);
//                    cout << d2 << "  " << s0 << "  " << t2 << "  " << pmf_p[n*(t2-1)+s0] << endl;
                    score2+=pmf_p[n*(t2-1)+s0];
                 }
              }
          }
    }
    return score2;
}

// calculate pmf energies for water molecules with other water molecules
double pmf_wat(double x0, double y0, double z0, double x, double y, double z, vector<double> pmf_w, double c1, double c2)
{
    double score0=0.0;
    double d0=0.0;
    d0=sqrt((x0-x)*(x0-x)+(y0-y)*(y0-y)+(z0-z)*(z0-z));
    if(d0<c1)  { score0+=4.0; }
    if(d0>=c1&&d0<=c2) 
    {
       int s0=int((d0-c1)/0.1);
       score0+=pmf_w[s0];
    }  
    return score0;
}


void swap(double *x, double *y)
{
    double temp;
    temp=*x;
    *x=*y;
    *y=temp;
}


// sort the eight vetex for the interpolation computation

void sort_vetex(double *gv_x, double *gv_y, double *gv_z, double *gv_pmf, double *gv_d, int n)
{
// sort x
    for(int i=0; i<n; i++)
    {
        for(int j=i+1; j<n; j++)
        {
            if(gv_x[j]<gv_x[i])
            {
               swap(&gv_x[j],&gv_x[i]);
               swap(&gv_y[j],&gv_y[i]);
               swap(&gv_z[j],&gv_z[i]);
               swap(&gv_pmf[j],&gv_pmf[i]);
               swap(&gv_d[j],&gv_d[i]);
               break;
            }
        }
    }
 
// sort y
    for(int i=0; i<n; i++)
    {
        for(int j=i+1; j<n; j++)
        {
           if(gv_y[j]<gv_y[i]&&gv_x[j]<=gv_x[i])
           {
               swap(&gv_x[j],&gv_x[i]);
               swap(&gv_y[j],&gv_y[i]);
               swap(&gv_z[j],&gv_z[i]);
               swap(&gv_pmf[j],&gv_pmf[i]);
               swap(&gv_d[j],&gv_d[i]);
               break;
           }
        }
    }   

//sort z
    for(int i=0; i<n; i++)
    {
        for(int j=i+1; j<n; j++)
        {
           if(gv_z[j]<gv_z[i]&&gv_x[j]<=gv_x[i]&&gv_y[j]<=gv_y[i])
           {
               swap(&gv_x[j],&gv_x[i]);
               swap(&gv_y[j],&gv_y[i]);
               swap(&gv_z[j],&gv_z[i]);
               swap(&gv_pmf[j],&gv_pmf[i]);
               swap(&gv_d[j],&gv_d[i]);
               break;
           }
        }
    }
/*
    for(int i=0; i<n; i++)
    {
       cout << gv_x[i] << "  " << gv_y[i] << "  " << gv_z[i] << endl;
    }
*/ 
}


// interpolation

double intpol_f(double E1, double E2, double r, double d)
{
   double Ex(0.0);
   Ex=(E2-E1)*d*(1.0/r)+E1;
   return Ex;
}

double get_pmf(double *gv_x, double *gv_y, double *gv_z, double *gv_pmf, double x, double y, double z, double r)
{
   double tmp1,tmp2,tmp3,tmp4,T1,T2,pmf;
   tmp1=tmp2=tmp3=tmp4=T1=T2=pmf=0.0;
   tmp2=intpol_f(gv_pmf[1],gv_pmf[3],r,(y-gv_y[0]));
   tmp1=intpol_f(gv_pmf[0],gv_pmf[2],r,(y-gv_y[0]));
   T1=intpol_f(tmp1,tmp2,r,(z-gv_z[0]));
   tmp4=intpol_f(gv_pmf[5],gv_pmf[7],r,(y-gv_y[4]));
   tmp3=intpol_f(gv_pmf[4],gv_pmf[6],r,(y-gv_y[4]));
   T2=intpol_f(tmp3,tmp4,r,(z-gv_z[4]));
   pmf=intpol_f(T1,T2,r,(x-gv_x[0]));
   return pmf;
}


string findMean(double x, double y, double z, vector<string> w_id, vector<double> w_x, vector<double> w_y, vector<double> w_z)
{
   double Dmin(100.0);
   string id;
   for(int i=0; i<w_x.size(); i++)
   {
      double d(0.0);
      d=sqrt((x-w_x[i])*(x-w_x[i])+(y-w_y[i])*(y-w_y[i])+(z-w_z[i])*(z-w_z[i]));
      if(d<Dmin)
      {
         Dmin=d;
         id=w_id[i];
      }
   }
   return id;
}

double getcoord(vector<double> x, vector<string> id, string a)
{
    for(int i=0; i<id.size(); i++)
    {
       if(a==id[i])
       {
          return x[i];
       }
    }
} 
