// find the closest pairs of water molecules 

#ifdef ANALWS_MAIN

#include <ccbg/CCMol.h>
#include <string>
#include <vector>
#include <ccwpmf2.h>

int main( int argc, char* argv[] )
{

	if (argc<4 || argc > 5) {
		cout << "\nUsage: " << FileName(argv[0]) << " ligand_file  wat1  wat2  \n" 
                     << " [ distance=4.0 ] " << endl;
		exit( EXIT_FAILURE );
	}

        double dis=4.0;

        if(argc==5) { dis=toDBL(argv[4]);  }

// get water locations 
        vector<double> W1_x, W1_y, W1_z;
        vector<string> W1_id;
        ifstream ifs(argv[2]);
        string line;
        while(!ifs.eof())
        {
           if(!getline(ifs,line))
           break;
           if(line.length()>30)
           {
              if(line.substr(0,4)=="ATOM")
              {
                 W1_x.push_back(toDBL(line.substr(30,8)));
                 W1_y.push_back(toDBL(line.substr(38,8)));
                 W1_z.push_back(toDBL(line.substr(46,8)));
                 W1_id.push_back(line.substr(8,3));
              }
           }
        }
        ifs.close();

        vector<double> W2_x, W2_y, W2_z;
        vector<string> W2_id;
        ifstream ifs2(argv[3]);
        string line2;
        while(!ifs2.eof())
        {
           if(!getline(ifs2,line2))
           break;
           if(line2.length()>30)
           {
              if(line2.substr(0,4)=="ATOM")
              {
                 W2_x.push_back(toDBL(line2.substr(30,8)));
                 W2_y.push_back(toDBL(line2.substr(38,8)));
                 W2_z.push_back(toDBL(line2.substr(46,8)));
                 W2_id.push_back(line2.substr(8,3));
              }
           }
        }
        ifs2.close();


// determine the water molecules located within 3A of the ligand
        CCMol m1;
        m1.ReadMolecule(argv[1]);
        vector<double> w1_x, w1_y, w1_z;
        vector<string> w1_id;
        for(int i=0; i <W1_x.size(); i++)
        {
           double dmin=10.0;
           double d0=0.0;
           double x0, y0, z0;
           x0=y0=z0=0.0;
           FOR_ATOMS_OF_MOL(a1,m1)
           {
              if(a1->IsHydrogen())  { continue; }
              else
              {
                 x0=a1->GetX();
                 y0=a1->GetY();
                 z0=a1->GetZ();
                 d0=sqrt((x0-W1_x[i])*(x0-W1_x[i])+(y0-W1_y[i])*(y0-W1_y[i])+(z0-W1_z[i])*(z0-W1_z[i]));
                 if(d0<dmin)
                 {
                     dmin=d0;
                 }
              }
           }
           if(dmin<=dis)
           {
              w1_x.push_back(W1_x[i]);
              w1_y.push_back(W1_y[i]);
              w1_z.push_back(W1_z[i]);
              w1_id.push_back(W1_id[i]);
           }
        }

        vector<double> w2_x, w2_y, w2_z;
        vector<string> w2_id;
        for(int i=0; i <W2_x.size(); i++)
        {
           double dmin=10.0;
           double d0=0.0;
           double x0, y0, z0;
           x0=y0=z0=0.0;
           FOR_ATOMS_OF_MOL(a1,m1)
           {
              if(a1->IsHydrogen())  { continue; }
              else
              {
                 x0=a1->GetX();
                 y0=a1->GetY();
                 z0=a1->GetZ();
                 d0=sqrt((x0-W2_x[i])*(x0-W2_x[i])+(y0-W2_y[i])*(y0-W2_y[i])+(z0-W2_z[i])*(z0-W2_z[i]));
                 if(d0<dmin)
                 {
                     dmin=d0;
                 }
              }
           }
           if(dmin<=dis)
           {
              w2_x.push_back(W2_x[i]);
              w2_y.push_back(W2_y[i]);
              w2_z.push_back(W2_z[i]);
              w2_id.push_back(W2_id[i]);
           }
        }

//  output the closest water molecule pairs


        while(w1_x.size()>0&&w2_x.size()>0)
        {
           string id1, id2, tmp;
           double d(0.0);
           for(int i=0; i<w1_x.size(); i++)
           {
              id1=w1_id[i];
              id2=findMean(w1_x[i],w1_y[i],w1_z[i],w2_id,w2_x,w2_y,w2_z);
              double x0, y0, z0;
              x0=y0=z0=0.0;
              x0=getcoord(w2_x,w2_id,id2);
              y0=getcoord(w2_y,w2_id,id2);
              z0=getcoord(w2_z,w2_id,id2);
              tmp=findMean(x0,y0,z0,w1_id,w1_x,w1_y,w1_z);
              if(tmp==id1)
              {
                d=sqrt((w1_x[i]-x0)*(w1_x[i]-x0)+(w1_y[i]-y0)*(w1_y[i]-y0)+(w1_z[i]-z0)*(w1_z[i]-z0));
                cout << "wat " << setw(3) << id1 << "  " << "wat " << setw(3) << id2 << "   distance: " <<  d << endl;
                for(int j=0; j<w2_x.size(); j++)
                {
                   if(w2_id[j]==id2)
                   {
                     w2_id.erase(w2_id.begin()+j);
                     w2_x.erase(w2_x.begin()+j);
                     w2_y.erase(w2_y.begin()+j);
                     w2_z.erase(w2_z.begin()+j);
                   }
                }
                
                for(int k=0; k<w1_x.size(); k++)
                {
                  if(w1_id[k]==id1)
                  {
                    w1_id.erase(w1_id.begin()+k);
                    w1_x.erase(w1_x.begin()+k);
                    w1_y.erase(w1_y.begin()+k);
                    w1_z.erase(w1_z.begin()+k);
                  }
                }
                 break;
              }
           
            }
     
        }


        return 0;
}

#endif
