// calculate the dG for each hydration site

#ifdef DG_MAIN

#include <ccbg/CCMol.h>
#include <string>
#include <vector>
#include <ccwpmf2.h>
#include <math.h>


double gv_x[8];
double gv_y[8];
double gv_z[8];
double gv_pmf[8];
double gv_d[8];

int main( int argc, char* argv[] )
{

        if (argc!=3) {
                cout << "\nUsage: " << FileName(argv[0]) << " hs_file  grid_file  \n" 
                << endl;
                exit( EXIT_FAILURE );
        }
   

// read the protein file, the grid box with calculated PMF energies, water locations and energies

   vector<double> g_x, g_y, g_z, g_pmf;          // grid box with PMF energies
   vector<double> w_x, w_y, w_z, w_pmf;          // water locations and energies
   ifstream fin(argv[1]);
   string line;
   getline(fin,line);
   while(!fin.eof())
   {
      if(line.substr(0,4)=="ATOM")
      {
         w_x.push_back(toDBL(line.substr(30,8)));
         w_y.push_back(toDBL(line.substr(38,8)));
         w_z.push_back(toDBL(line.substr(46,8)));
      }
      if(!getline(fin,line))
      break;     
   }   
   fin.close();

   ifstream fin2(argv[2]);
   vector<string> vs0;
   string str;
   while(!fin2.eof())
   {
      if(!getline(fin2,str))
      break;
      Tokenize(vs0,str);
      g_x.push_back(toDBL(vs0[1]));
      g_y.push_back(toDBL(vs0[2]));
      g_z.push_back(toDBL(vs0[3]));
      g_pmf.push_back(toDBL(vs0[4]));   
   }
   fin2.close();

   int grid_num=g_x.size();

// read PMF parameter file of the water oxygen atom
   ifstream ifs2("PARA_WAT.TXT");
   string line2;
   vector<string> vs2;
   vector<double> pmf_w;
   double c3(0.0), c4(0.0);
   while(!ifs2.eof())
   {
      if(!getline(ifs2,line2))
      break;
      Tokenize(vs2,line2);
      if(vs2[0]=="Distance")
      {
         c3=toDBL(vs2[1]);
         c4=toDBL(vs2[2]);
      }
      double ff;
      if(sscanf(vs2[0].c_str(),"%lf",&ff)==1)
      {
         pmf_w.push_back(toDBL(vs2[3]));
      }
   }
   ifs2.close();

   
// determine the energy for each water molecule in the binding site
   for(int i=0; i<w_x.size(); i++)
   {
          vector<double> t_x, t_y, t_z, t_pmf, t_d;
   	  for(int j=0; j<g_x.size(); j++)
   	  {
   	  	 double d(0.0);    
   	  	 if(abs(w_x[i]-g_x[j])<0.5&&abs(w_y[i]-g_y[j])<0.5&&abs(w_z[i]-g_z[j])<0.5)
   	  	 {
                    d=sqrt((w_x[i]-g_x[j])*(w_x[i]-g_x[j])+(w_y[i]-g_y[j])*(w_y[i]-g_y[j])+(w_z[i]-g_z[j])*(w_z[i]-g_z[j]));
//                    cout << i << "  " << g_x[j] << "  " << g_y[j] << "  " << g_z[j] << endl;
   	  	    t_x.push_back(g_x[j]);
   	  	    t_y.push_back(g_y[j]);
   	  	    t_z.push_back(g_z[j]);
   	  	    t_pmf.push_back(g_pmf[j]);
   	  	    t_d.push_back(d);
   	  	 }
          }

//          cout << i << "  " << t_x.size()  ;
          
// calculate the pmf energies of water molecules by interpolation

          if(t_x.size()==1)
          {
             w_pmf.push_back(t_pmf[0]);
          }

          if(t_x.size()==2)
          {
                 for(int k=0; k<2; k++)
                 {
                    gv_x[k]=t_x[k];
                    gv_y[k]=t_y[k];
                    gv_z[k]=t_z[k];
                    gv_pmf[k]=t_pmf[k];
                    gv_d[k]=t_d[k];
                 }
                 double pmf(0.0);
                 sort_vetex(gv_x, gv_y, gv_z, gv_pmf, gv_d,2);
                 if((gv_x[0]==gv_x[1])&&(gv_y[0]==gv_y[1]))
                 {
                    pmf=intpol_f(gv_pmf[0],gv_pmf[1],0.5,(w_z[i]-gv_z[0]));
                    w_pmf.push_back(pmf);
                 }
                 if((gv_x[0]==gv_x[1])&&(gv_z[0]==gv_z[1]))
                 {
                    pmf=intpol_f(gv_pmf[0],gv_pmf[1],0.5,(w_y[i]-gv_y[0]));
                    w_pmf.push_back(pmf);
                 }
                 if((gv_z[0]==gv_z[1])&&(gv_y[0]==gv_y[1]))
                 {
                    pmf=intpol_f(gv_pmf[0],gv_pmf[1],0.5,(w_x[i]-gv_x[0]));
                    w_pmf.push_back(pmf);
                 }
          }

          if(t_x.size()==4)
          {
                 for(int k=0; k<4; k++)
                 {
                    gv_x[k]=t_x[k];
                    gv_y[k]=t_y[k];
                    gv_z[k]=t_z[k];
                    gv_pmf[k]=t_pmf[k];
                    gv_d[k]=t_d[k];
                 }
                 double pmf(0.0);
                 sort_vetex(gv_x, gv_y, gv_z, gv_pmf, gv_d,4);
                 double tmp1(0.0), tmp2(0.0);
                 if((gv_x[0]==gv_x[1])&&(gv_x[0]==gv_x[2])&&(gv_x[0]==gv_x[3]))
                 {
                    tmp1=intpol_f(gv_pmf[0],gv_pmf[2],0.5,(w_y[i]-gv_y[0]));
                    tmp2=intpol_f(gv_pmf[1],gv_pmf[3],0.5,(w_y[i]-gv_y[0]));
                    pmf=intpol_f(tmp1,tmp2,0.5,(w_z[i]-gv_z[0]));
                    w_pmf.push_back(pmf);
                 }
                 if((gv_y[0]==gv_y[1])&&(gv_y[0]==gv_y[2])&&(gv_y[0]==gv_y[3]))
                 {
                    tmp1=intpol_f(gv_pmf[0],gv_pmf[2],0.5,(w_x[i]-gv_x[0]));
                    tmp2=intpol_f(gv_pmf[1],gv_pmf[3],0.5,(w_x[i]-gv_x[0]));
                    pmf=intpol_f(tmp1,tmp2,0.5,(w_z[i]-gv_z[0]));
                    w_pmf.push_back(pmf);
                 }
                 if((gv_z[0]==gv_z[1])&&(gv_z[0]==gv_z[2])&&(gv_z[0]==gv_z[3]))
                 {
                    tmp1=intpol_f(gv_pmf[0],gv_pmf[2],0.5,(w_x[i]-gv_x[0]));
                    tmp2=intpol_f(gv_pmf[1],gv_pmf[3],0.5,(w_x[i]-gv_x[0]));
                    pmf=intpol_f(tmp1,tmp2,0.5,(w_y[i]-gv_y[0]));
                    w_pmf.push_back(pmf);
                 }
          }

          if(t_x.size()==8)
          {
                 for(int k=0; k<8; k++)
                 {
                    gv_x[k]=t_x[k];
                    gv_y[k]=t_y[k];
                    gv_z[k]=t_z[k];
                    gv_pmf[k]=t_pmf[k];
                    gv_d[k]=t_d[k];
                 }
                 sort_vetex(gv_x, gv_y, gv_z, gv_pmf, gv_d,8);
                 double pmf(0.0);
                 pmf=get_pmf(gv_x,gv_y,gv_z,gv_pmf,w_x[i],w_y[i],w_z[i],0.5);
                 w_pmf.push_back(pmf);
          }

   	  t_x.clear();
   	  t_y.clear();
   	  t_z.clear();
   	  t_pmf.clear();
   	  t_d.clear();
   }

// calculate the interation energy from other water molecules
//   CCMol m1;
//   m1.ReadMolecule(argv[1]);
   for(int i=0; i<w_x.size(); i++)
   {
      double pmf(0.0);
      double d(0.0);
      for(int l=0; l<w_x.size(); l++)
      {
            if(i!=l)
            {
               d=sqrt((w_x[i]-w_x[l])*(w_x[i]-w_x[l])+(w_y[i]-w_y[l])*(w_y[i]-w_y[l])+(w_z[i]-w_z[l])*(w_z[i]-w_z[l]));
               pmf+=pmf_wat(w_x[i],w_y[i],w_z[i],w_x[l],w_y[l],w_z[l],pmf_w,c3,c4);              // energy from other water molecules
            }
      }
      w_pmf[i]+=pmf;
//      cout << i << " " << pmf << " " << w_pmf[i] << endl;
   }




  ofstream fout("wsite.log");
  for(int l=0; l<w_x.size(); l++)
  {
         fout <<  setw(3) << l+1 << "  "  << setw(8) << fixed << setprecision(3) <<  w_x[l] << setw(8) << fixed << setprecision(3) << w_y[l] << setw(8) << fixed << setprecision(3) << w_z[l] << "  " << setw(5) << fixed << setprecision(2) << w_pmf[l] << endl;
  }
  fout.close();

  return 0;

}


#endif
