// find water molecules within 4A from both ligand and protein

#ifdef GETWAT_MAIN

#include <ccbg/CCMol.h>
#include <string>
#include <vector>
#include <ccwpmf2.h>

int main( int argc, char* argv[] )
{

	if (argc<4 || argc > 5) {
		cout << "\nUsage: " << FileName(argv[0]) << " protein_file ligand_file wat_file  \n" 
                     << " [ distance=4.0 ] " << endl;
		exit( EXIT_FAILURE );
	}

        double dis=4.0;

        if(argc==5) { dis=toDBL(argv[4]);  }

// get water locations 
        vector<double> W1_x, W1_y, W1_z;
        vector<string> W1_id;
        ifstream ifs(argv[3]);
        string line;
        while(!ifs.eof())
        {
           if(!getline(ifs,line))
           break;
           if(line.length()>30)
           {
              if(line.substr(0,4)=="ATOM")
              {
                 W1_x.push_back(toDBL(line.substr(30,8)));
                 W1_y.push_back(toDBL(line.substr(38,8)));
                 W1_z.push_back(toDBL(line.substr(46,8)));
                 W1_id.push_back(line.substr(8,3));
              }
           }
        }
        ifs.close();

// determine the water molecules located within 4A of the ligand
        CCMol m1, m2;
        m1.ReadMolecule(argv[1]);
        m2.ReadMolecule(argv[2]);
        vector<double> w1_x, w1_y, w1_z;
        vector<string> w1_id;
        for(int i=0; i <W1_x.size(); i++)
        {
           double d0=0.0;
           double x0, y0, z0;
           x0=y0=z0=0.0;
           FOR_ATOMS_OF_MOL(a2,m2)
           {
              if(a2->IsHydrogen())  { continue; }
              else
              {
                 x0=a2->GetX();
                 y0=a2->GetY();
                 z0=a2->GetZ();
                 d0=sqrt((x0-W1_x[i])*(x0-W1_x[i])+(y0-W1_y[i])*(y0-W1_y[i])+(z0-W1_z[i])*(z0-W1_z[i]));
                 if(d0<dis)
                 {
                     w1_x.push_back(W1_x[i]);
                     w1_y.push_back(W1_y[i]);
                     w1_z.push_back(W1_z[i]);
                     w1_id.push_back(W1_id[i]);
                     break;
                 }
              }
           }
        }

// determine the water molecules located within 4A of the protein
        vector<double> w2_x, w2_y, w2_z;
        vector<string> w2_id;
        for(int i=0; i <w1_x.size(); i++)
        {
           double d0=0.0;
           double x0, y0, z0;
           x0=y0=z0=0.0;
           FOR_RESIDUES_OF_MOL(r1,m1)
           {
              int flag=0;
              FOR_ATOMS_OF_RESIDUE(a1,&*r1)
              {
                 if(a1->IsHydrogen())  { continue; }
                 else
                 {
                    x0=a1->GetX();
                    y0=a1->GetY();
                    z0=a1->GetZ();
                    d0=sqrt((x0-w1_x[i])*(x0-w1_x[i])+(y0-w1_y[i])*(y0-w1_y[i])+(z0-w1_z[i])*(z0-w1_z[i]));
                    if(d0<dis)
                    {
                        cout << d0 << " " << w1_x[i] << "  " <<  w1_y[i] << "  " << w1_z[i] << endl;
                        w2_x.push_back(w1_x[i]);
                        w2_y.push_back(w1_y[i]);
                        w2_z.push_back(w1_z[i]);
                        w2_id.push_back(w1_id[i]);
                        flag=1;
                        break;
                    }
                 }
             }
             if(flag==1)
             {
                 break;
             }
          }
       }

      cout << w2_x.size() << endl;

//  output the water molecules
       
       ofstream fout("wat.pdb");
       fout << "REMARK   WATER MOLECULES" << endl;
       for(int i=0; i<w2_x.size(); i++)
       {
          fout <<"ATOM  " << setw(5) << i+1 << "  O   HOH   " << setw(3) << i+1 << "    " << setw(8) << fixed << setprecision(3) <<  w2_x[i] << setw(8) << fixed << setprecision(3) << w2_y[i] << setw(8) << fixed << setprecision(3) << w2_z[i] << "  1.00  0.00" << "          O" << endl;
       }
       fout.close();

        return 0;
}

#endif
