// mean shift

#ifdef WSITEM_MAIN

#include <ccbg/CCMol.h>
#include <string>
#include <vector>
#include <ccwpmf2.h>

double gv_x[8];
double gv_y[8];
double gv_z[8];
double gv_pmf[8];
double gv_d[8];

int main( int argc, char* argv[] )
{

	if (argc<4 || argc>8) {
		cout << "\nUsage: " << FileName(argv[0]) << " protein ligand apo/holo\n"
                     << " [ D=4.0 cutoff=5.0 radius=2.0 N=10 ]"   << endl;
		exit( EXIT_FAILURE );
	}

	string PDB1 = argv[1];
        string MOL = argv[2];
        string state = argv[3];
        double D = 4.0;
        double cutoff = 5.0;
        double radius = 2.0;
        double N = 10;
 
        if (argc>=5) { D = toDBL(argv[4]); }
        if (argc>=6) { cutoff = toDBL(argv[5]);  }
        if (argc>=7) { radius = toDBL(argv[6]);  }
        if (argc==8) { N = toDBL(argv[7]); }

// determine the boundaries of the ligand atoms
        CCMol m2;
        m2.ReadMolecule(MOL);
        vector<double> R_X, R_Y, R_Z;
        FOR_ATOMS_OF_MOL(a2,m2) 
        {
             if(a2->IsHydrogen())  {continue;}
             else
             {
                  R_X.push_back(a2->GetX());
                  R_Y.push_back(a2->GetY());
                  R_Z.push_back(a2->GetZ());
             }
                  continue;
        }

        double x_min, x_max, y_min, y_max, z_min, z_max;
        x_min=x_max=R_X[0];
        y_min=y_max=R_Y[0];
        z_min=z_max=R_Z[0];
        for(int i=1; i<R_X.size(); i++)
        {
           if(R_X[i]<=x_min)
           {
              x_min=R_X[i];
           }
           if(R_Y[i]<=y_min)
           {
              y_min=R_Y[i];
           }
           if(R_Z[i]<=z_min)
           {
              z_min=R_Z[i];
           } 
        }
        for(int i=1; i<R_X.size(); i++)
        {
           if(R_X[i]>=x_max)
           {
              x_max=R_X[i];
           }
           if(R_Y[i]>=y_max)
           {
              y_max=R_Y[i];
           }
           if(R_Z[i]>=z_max)
           {
              z_max=R_Z[i];
           }
        }
          
// place grid box, stepwise=0.5A. 

        vector<double> g_x;
        vector<double> g_y;
        vector<double> g_z;
        double x, y, z;
        x=y=z=0.0;
        int a, b, c;        // length, width and height of the grid box
        a=x_max-x_min+2*D;
        b=y_max-y_min+2*D;
        c=z_max-z_min+2*D;
        int n_x, n_y, n_z;
        n_x=int(a/0.5)+1;
        n_y=int(b/0.5)+1;
        n_z=int(c/0.5)+1;
        for(int i=0; i<n_x; i++)
        {
           for(int j=0; j<n_y; j++)
           {
              for(int k=0; k<n_z; k++)
              {
                 g_x.push_back(x_max+D-0.5*i);
                 g_y.push_back(y_max+D-0.5*j);
                 g_z.push_back(z_max+D-0.5*k);
              }
           }
        }

    
      int grid_num=g_x.size();

// read PMF parameter file of the protein atoms
      ifstream ifs("PARA_PRO.TXT");
      string line;
      vector<string> vs;
      vector<double> pmf_p;
      double c1(0.0), c2(0.0);
      while(!ifs.eof())
      {
         if(!getline(ifs,line))
         break;
         Tokenize(vs,line);
         if(vs[0]=="Distance")
         {
            c1=string2double(vs[1]);
            c2=string2double(vs[2]);
         }
         double f;
         if(sscanf(vs[0].c_str(),"%lf",&f)==1)
         {
            pmf_p.push_back(string2double(vs[3]));
         } 
      }
      ifs.close();

// read PMF parameter file of the water oxygen atom
      ifstream ifs2("PARA_WAT.TXT");
      string line2;
      vector<string> vs2;
      vector<double> pmf_w;
      double c3(0.0), c4(0.0);
      while(!ifs2.eof())
      {
         if(!getline(ifs2,line2))
         break;
         Tokenize(vs2,line2);
         if(vs2[0]=="Distance")
         {
            c3=string2double(vs2[1]);
            c4=string2double(vs2[2]);
         }
         double ff;
         if(sscanf(vs2[0].c_str(),"%lf",&ff)==1)
         {
            pmf_w.push_back(string2double(vs2[3]));
         } 
      }
      ifs2.close();

      vector<double> pmf_l;
      double c5(0.0), c6(0.0);
      if(state=="holo")
      {
//  read PMF parameter file of the ligand atom
        ifstream ifs3("PARA_LIG.TXT");
        string line3;
        vector<string> vs3;
        while(!ifs3.eof())
        {
           if(!getline(ifs3,line3))
           break;
           Tokenize(vs3,line3);
           if(vs3[0]=="Distance")
           {
              c5=string2double(vs3[1]);
              c6=string2double(vs3[2]);
           }
           double FF;
           if(sscanf(vs3[0].c_str(), "%lf",&FF)==1)
           {
              pmf_l.push_back(string2double(vs3[3]));
           }
         }
         ifs3.close();
       }

// read protein file
        
        CCMol m1;
        m1.ReadMolecule(PDB1);

        double x1, y1, z1;
        double x2, y2, z2;
        double *pmf_sum;
        pmf_sum = new double[grid_num];
        for(int m=0; m<grid_num; m++)
        {
           pmf_sum[m]=0.0;
        }
//        cout << "water" << "   " << "pmf" << endl;

//  calculate pmf for each grid with protein atoms
        vector<int> c_id;
        vector<double> c_pmf;
        ofstream fo("grid.txt");
        for(int i=0; i<grid_num; i++)
        {
            x1=g_x[i];
            y1=g_y[i];
            z1=g_z[i];

            if(state=="apo")
            {
              pmf_sum[i]+=pmf_pro(m1,x1,y1,z1,pmf_p,c1,c2);              
              fo << i << " " << x1 << "  " << y1 << "  " << z1 << "  " << pmf_sum[i] << endl;
            }

            if(state=="holo")
            {
              pmf_sum[i]+=pmf_lig(m2,x1,y1,z1,pmf_l,c5,c6);
            }         

//  keep grids with pmfs lower than cutoff
            if(pmf_sum[i]<cutoff)
            {
               c_id.push_back(i);
            }
        }

        fo.close();

//        cout << c_id.size() << endl;
        vector<int> flag;
        for(int i=0; i<grid_num; i++)
        {
            flag.push_back(0);
        }

// clustering the grids based on the mean shift algorithm
// take a random grid as the initial center and calculate the mean shift of all grids with a sphere of radius as H(i.e.H=2)
// w(xi)=weight defined as 1/dE (dE=|Emin-E|)
// Mh(x)=[sum(w(xi)(xi-x)]/[sum(w(xi))]

         vector<double> out_x;
         vector<double> out_y;
         vector<double> out_z;          // coordinates of cluster centers
         ofstream fout("wsite.pdb");   // output the water site pdb file
         ofstream fout2("wsite.log");   // output the details for water wsites
         vector<double>  cls_x, cls_y, cls_z;  //  coordinates of the cluster center
         vector<double>  cls_pmf;         //  energy of the cluster center
         int clsNum=1;
         int g0_num=0;
//         double radius=2.0;
         while(c_id.size()!=0)
         {
            double t_x, t_y, t_z;       // coordinates of the intermediate cluster center
            t_x=t_y=t_z=0.0;
            double t_pmf(0.0);
            g0_num=c_id.size();
            for(int k=0; k<c_id.size(); k++)
            {
               t_x=g_x[c_id[k]];
               t_y=g_y[c_id[k]];
               t_z=g_z[c_id[k]];
               t_pmf=pmf_sum[c_id[k]];
//               cout << c_id[k] << " " << t_x << " " << t_y << " " <<t_z << endl;

loop:          if(c_id.size()>0)
               {
                  vector<int> tmp_cls;
                  double sx,sy,sz;
                  sx=sy=sz=0.0;
                  double dd=0.0;
                  for(int l=0; l<c_id.size(); l++)
                  {
                     dd=sqrt((t_x-g_x[c_id[l]])*(t_x-g_x[c_id[l]])+(t_y-g_y[c_id[l]])*(t_y-g_y[c_id[l]])+(t_z-g_z[c_id[l]])*(t_z-g_z[c_id[l]]));
                     if(dd<=radius)
                     {
//                        cout << c_id[l] << " " << g_x[c_id[l]] << endl;                      
                        tmp_cls.push_back(c_id[l]);
                     }
                  }
                   
                  int n=0;
                  n=tmp_cls.size();
//                  cout << w.size() << endl;
                  if(tmp_cls.size()<N)
                  {
                     for(int l=0; l<tmp_cls.size(); l++)
                     {
/*                        for(int m=0; m<c_id.size(); m++)
                        {
                           if(c_id[m]==tmp_cls[l])
                           {
                              c_id.erase(c_id.begin()+m);
                           }
                        }
*/
                     }

                     tmp_cls.clear();
                  }
              
                  
                  if(tmp_cls.size()>=N)
                  {
                     for(int m=0; m<tmp_cls.size(); m++)
                     {
                        sx+=(g_x[tmp_cls[m]]-t_x);
                        sy+=(g_y[tmp_cls[m]]-t_y);
                        sz+=(g_z[tmp_cls[m]]-t_z);
                        flag[tmp_cls[m]]=clsNum;
                     }
                     sx=double(sx/n);
                     sy=double(sy/n);
                     sz=double(sz/n);
//                     cout << sx << " " << sy << " " << sz << endl;

               
                     if(sx*sx+sy*sy+sz*sz<0.02)      // output the cluster
                     {
                        if(cls_x.size()>0)          // merge cluster
                        { 
                           int merge=0;
                           for(int p=0; p<cls_x.size(); p++)
                           {
                              double d2=0.0;
                              d2=sqrt((t_x-cls_x[p])*(t_x-cls_x[p])+(t_y-cls_y[p])*(t_y-cls_y[p])+(t_z-cls_z[p])*(t_z-cls_z[p]));
                              t_pmf=pmf_pro(m1,t_x,t_y,t_z,pmf_p,c1,c2);
                              if(d2<radius)
                              {
                                 merge=1;
                                 for(int i=0; i<tmp_cls.size(); i++)
                                 {
                                    flag[tmp_cls[i]]=p+1; 
                                 }
/*                                 if(t_pmf<cls_pmf[p])
                                 {
                                    cls_x[p]=t_x;
                                    cls_y[p]=t_y;
                                    cls_z[p]=t_z;
                                    cls_pmf[p]=t_pmf;
                                 }
*/ 
                               break;
                              }
                           }
                           if(merge==0)
                           {
//                               cout << "Cls " << t_x << "  " << t_y << "  " << t_z << endl;
                               cls_x.push_back(t_x);
                               cls_y.push_back(t_y);
                               cls_z.push_back(t_z);
                               cls_pmf.push_back(t_pmf);
                               for(int i=0; i<tmp_cls.size(); i++)
                               {
                                    flag[tmp_cls[i]]=clsNum;
                               }
                               clsNum++;
                           }
                        }
                        else
                        {
//                                 cout << "Cls2 " << t_x << "  " << t_y << "  " << t_z << endl;
                                 cls_x.push_back(t_x);
                                 cls_y.push_back(t_y);
                                 cls_z.push_back(t_z);
                                 cls_pmf.push_back(t_pmf);
                                 for(int i=0; i<tmp_cls.size(); i++)
                                 {
                                    flag[tmp_cls[i]]=clsNum;
                                 }
                                 clsNum++;
                        }
                        for(int i=0; i<c_id.size(); i++)
                        {
                           if(flag[c_id[i]]>0)
                           {
                              c_id.erase(c_id.begin()+i);
                           }
                        }
                        tmp_cls.clear();
                     }

                     else
                     {
                        t_x=t_x+sx;
                        t_y=t_y+sy;
                        t_z=t_z+sz;
//                        cout << t_x << "  " << t_y << " " << t_z << endl;
                        t_pmf=pmf_pro(m1,t_x,t_y,t_z,pmf_p,c1,c2);
                        tmp_cls.clear();
                        goto loop;
                     }
                 }
            }
            
       }
       if(c_id.size()==g0_num)
       {
          break;
       }
    }

// output the cluster centers as pdb file
            fout << "REMARK   PREDICTED WATER SITE" << endl;
            for(int i=0; i<cls_x.size(); i++)
            {
                fout <<"ATOM  " << setw(5) << i+1 << "  O   HOH   " << setw(3) << i+1 << "    " << setw(8) << fixed << setprecision(3) <<  cls_x[i] << setw(8) << fixed << setprecision(3) << cls_y[i] << setw(8) << fixed << setprecision(3) << cls_z[i] << "  1.00  0.00" << "          O" << endl;
            }

            fout.close();

// calculate the pmf energy of predicted water by linear interpolation
            vector<double> w_pmf; 
            for(int i=0; i<cls_x.size(); i++)
            {
                vector<double> t_x, t_y, t_z, t_pmf, t_d;
                for(int l=0; l<grid_num; l++)
                {
                    double d(0.0);
                    if(abs(cls_x[i]-g_x[l])<0.5&&abs(cls_y[i]-g_y[l])<0.5&&abs(cls_z[i]-g_z[l])<0.5)
                    {
                       d=sqrt((cls_x[i]-g_x[l])*(cls_x[i]-g_x[l])+(cls_y[i]-g_y[l])*(cls_y[i]-g_y[l])+(cls_z[i]-g_z[l])*(cls_z[i]-g_z[l]));
                       t_x.push_back(g_x[l]);
                       t_y.push_back(g_y[l]);
                       t_z.push_back(g_z[l]);
                       t_pmf.push_back(pmf_sum[l]);
                       t_d.push_back(d);
                    }
                }

                if(t_x.size()==8)
                {
                   for(int k=0; k<8; k++)
                   {
                      gv_x[k]=t_x[k];
                      gv_y[k]=t_y[k];
                      gv_z[k]=t_z[k];
                      gv_pmf[k]=t_pmf[k];
                      gv_d[k]=t_d[k];
                   }
                   sort_vetex(gv_x, gv_y, gv_z, gv_pmf, gv_d,8);
                   double pmf(0.0);
                   pmf=get_pmf(gv_x,gv_y,gv_z,gv_pmf,cls_x[i],cls_y[i],cls_z[i],0.5);
                   w_pmf.push_back(pmf);
                }
 
                if(t_x.size()==1)
                {
                    w_pmf.push_back(t_pmf[0]);
                }

                if(t_x.size()==2)
                {
                  for(int k=0; k<2; k++)
                  {
                    gv_x[k]=t_x[k];
                    gv_y[k]=t_y[k];
                    gv_z[k]=t_z[k];
                    gv_pmf[k]=t_pmf[k];
                    gv_d[k]=t_d[k];
                  }
                  double pmf(0.0);
                  sort_vetex(gv_x, gv_y, gv_z, gv_pmf, gv_d,2);
                  if(gv_x[0]==gv_x[1]&&gv_y[0]==gv_y[1])
                  {
                    pmf=intpol_f(gv_pmf[0],gv_pmf[1],0.5,(cls_z[i]-gv_z[0]));
                    w_pmf.push_back(pmf);
                  }
                  if(gv_x[0]==gv_x[1]&&gv_z[0]==gv_z[1])
                  {
                    pmf=intpol_f(gv_pmf[0],gv_pmf[1],0.5,(cls_y[i]-gv_y[0]));
                    w_pmf.push_back(pmf);
                  }
                  if(gv_z[0]==gv_z[1]&&gv_y[0]==gv_y[1])
                  {
                    pmf=intpol_f(gv_pmf[0],gv_pmf[1],0.5,(cls_x[i]-gv_x[0]));
                    w_pmf.push_back(pmf);
                  }
                }

                if(t_x.size()==4)
                {
                  for(int k=0; k<4; k++)
                  {
                    gv_x[k]=t_x[k];
                    gv_y[k]=t_y[k];
                    gv_z[k]=t_z[k];
                    gv_pmf[k]=t_pmf[k];
                    gv_d[k]=t_d[k];
                  }
                  double pmf(0.0);
                  sort_vetex(gv_x, gv_y, gv_z, gv_pmf, gv_d,4);
                  double tmp1(0.0), tmp2(0.0);
                  if((gv_x[0]==gv_x[1])&&(gv_x[0]==gv_x[2])&&(gv_x[0]==gv_x[3]))
                  {
                    tmp1=intpol_f(gv_pmf[0],gv_pmf[2],0.5,(cls_y[i]-gv_y[0]));
                    tmp2=intpol_f(gv_pmf[1],gv_pmf[3],0.5,(cls_y[i]-gv_y[0]));
                    pmf=intpol_f(tmp1,tmp2,0.5,(cls_z[i]-gv_z[0]));
                     w_pmf.push_back(pmf);
                  }
                  if((gv_y[0]==gv_y[1])&&(gv_y[0]==gv_y[2])&&(gv_y[0]==gv_y[3]))
                  {
                    tmp1=intpol_f(gv_pmf[0],gv_pmf[2],0.5,(cls_x[i]-gv_x[0]));
                    tmp2=intpol_f(gv_pmf[1],gv_pmf[3],0.5,(cls_x[i]-gv_x[0]));
                    pmf=intpol_f(tmp1,tmp2,0.5,(cls_z[i]-gv_z[0]));
                     w_pmf.push_back(pmf);
                  }
                  if((gv_z[0]==gv_z[1])&&(gv_z[0]==gv_z[2])&&(gv_z[0]==gv_z[3]))
                  {
                    tmp1=intpol_f(gv_pmf[0],gv_pmf[2],0.5,(cls_x[i]-gv_x[0]));
                    tmp2=intpol_f(gv_pmf[1],gv_pmf[3],0.5,(cls_x[i]-gv_x[0]));
                    pmf=intpol_f(tmp1,tmp2,0.5,(cls_y[i]-gv_y[0]));
                    w_pmf.push_back(pmf);
                  }
              }


                t_x.clear();
                t_y.clear();
                t_z.clear();
                t_pmf.clear();
                t_d.clear();
            }



// output the cluster coordinates and energy
            for(int i=0; i<cls_x.size(); i++)
            {
               fout2 << "Cluster " << setw(3) << i+1 << "  " << setw(8) << fixed << setprecision(3) <<  cls_x[i] << setw(8) << fixed << setprecision(3) << cls_y[i] << setw(8) << fixed << setprecision(3) << cls_z[i]  << setw(8) << fixed << setprecision(3) << w_pmf[i] << endl;
            }

            return 0;
}

#endif
