#include <ccbg/CCChain.h>
#include <ccbg/CCMol.h>

CCChain::CCChain()
    : m_mol( NULL ), m_idx( 0 ), m_chnid( '-' )
                     //m_vresidue

{
    CCBase::CCBase();
}

CCChain::~CCChain()
{
    CCBase::DestroyGD();

    CCResidueIter i = m_vresidue.begin();
    for ( ; i != m_vresidue.end(); ++i ) {
        if ( *i ) {
            ( *i )->SetChain( NULL );
        }
    }
    m_vresidue.clear();
}

CCChain::CCChain( const CCChain& src )
    : m_mol( src.m_mol ), m_idx( src.m_idx ), m_chnid( src.m_chnid ),
      m_vresidue( src.m_vresidue )
{
    CCBase::Clone( src );
}

CCChain& CCChain::operator =( const CCChain& src )
{
    if ( this != &src ) {
        CCBase::Clone( src );
        m_mol = src.m_mol;
        m_idx = src.m_idx;
        m_chnid = src.m_chnid;
        m_vresidue = src.m_vresidue;
    }
    return *this;
}

CCResidue* CCChain::GetResidue( int idx ) const
{
    return ( idx <1 || idx> GetNumRes() )
         ? NULL
         : m_vresidue[idx - 1];
}

CCResidue* CCChain::BeginResidue( CCResidueIter& i )
{
    i = m_vresidue.begin();
    return ( i == m_vresidue.end() ? NULL : *i );
}

CCResidue* CCChain::NextResidue( CCResidueIter& i )
{
    i++;
    return ( i == m_vresidue.end() ? NULL : *i );
}

void CCChain::AddResidue( CCResidue* res )
{
    if ( res != NULL ) {
        res->SetChain( this );
        m_vresidue.push_back( res );
    }
}

void CCChain::RemoveResidue( CCResidue* res )
{
    if ( res != NULL ) {
        for ( int i = 0; i < m_vresidue.size(); ++i ) {
            if ( m_vresidue[i] == res ) {
                res->SetChain( NULL );
                m_vresidue.erase( m_vresidue.begin() + i );
                break;
            }
        }
    }
}

