/** 
* @file 
* @brief interface for handling error
* @author Yuan ZHAO, Tiejun CHENG
* @date 2008/09/09
* @version 1.0.1 
*
*/

#include <ccbg/CCError.h>

/** @brief global error handler */
CCErrorLog gErrorLog;

/** @brief format and output an error message */
string CCErrorLog::CCError::message() const
{
    string msg;
    msg += "-+-+-+-+-+-+-+-+-+-  CCErrorLog  -+-+-+-+-+-+-+-+-+-\n";

    if ( m_level == ccWarning ) {
        msg += "ccWarning:";
    }
    else if ( m_level == ccFatal ) {
        msg += "  ccFatal:";
    }
    else {
        msg += "ccUnknown:";
    }

    if ( m_method.size() ) {
        msg += " in FUNCTION " + m_method + "\n";
    }

    msg += m_errorMsg;

    return msg;
}

/** 
* @brief constructor 
*
* error messages can be directed to standard output and/or a log file,
* useful if one wanna direct the error messages to both destinations
*/
CCErrorLog::CCErrorLog( ostream* oss/*=&clog*/, const string& logFile/*=""*/ )
    : m_oss( NULL ), m_ofs( NULL )
{
    m_oss = oss;
    if ( !logFile.empty() ) {
        static ofstream ofs( logFile.c_str() ); // must be static
        if ( !ofs ) {
            printf( "-+-+-+-+-+-+-+-+-+-  Oops  -+-+-+-+-+-+-+-+-+-\n" );
            printf( "Unable to open file '%s'\n", logFile.c_str() );
            printf( "Please make sure it exist.\n" );
            exit( EXIT_FAILURE );
        }
        m_ofs = &ofs;
    }
}

/** @brief destructor */
CCErrorLog::~CCErrorLog()
{
    //
}

/** @brief throw an error */
void CCErrorLog::Throw( CCError& error )
{
    if ( m_oss && m_oss->good() ) {
        *m_oss << error;
    }
    if ( m_ofs && m_ofs->good() ) {
        *m_ofs << error;
    }
}

/** 
* @brief throw an error 
* @param[in] method function throwing an error
* @param[in] errorMsg error message string
* @param[in] level error message level
*/
void CCErrorLog::Throw( const string& method, const string& errorMsg,
                        ccMsgLevel   level /*= ccWarning*/ )
{
    if ( errorMsg.size() ) {
        CCError error( method, errorMsg, level );
        Throw( error );
    }
}

