#include <ccbg/CCIter.h>
#include <ccbg/CCMol.h>

// 汾ܷඨУػ汾޷
template<class TParent, class TChild, typename Container>
void* ParentChildIter<TParent, TChild, Container>::operator~() const
{
	return NULL;
}

/** Iterate over all atoms of CCMol **/

// ģ๹캯ػ
// keyword template<> is needed for g++ compatibility 
template<>
void CCMolAtomIter::Init(CCMol* parent)
{
	m_parent = parent;
	m_child = m_parent->BeginAtom( m_iter );
}

// ģԱػ
template<>
CCMolAtomIter CCMolAtomIter::operator++()
{
	m_child = m_parent->NextAtom( m_iter );
	return *this;
}

/** class CCMolBondIter **/

template<>
void CCMolBondIter::Init( CCMol* parent )
{
    m_parent = parent;
    m_child = m_parent->BeginBond( m_iter );
}

template<>
CCMolBondIter CCMolBondIter::operator++()
{
    m_child = m_parent->NextBond( m_iter );
    return *this;
}

/** class CCAtomAtomIter **/

template<>
void CCAtomAtomIter::Init( CCAtom* parent )
{
	m_parent = parent;
	m_child = m_parent->BeginNbrAtom( m_iter );
}

template<>
CCAtomAtomIter CCAtomAtomIter::operator++()
{
    m_child = m_parent->NextNbrAtom( m_iter );
    return *this;
}

// ģԱػ
template<>
void* CCAtomAtomIter::operator~() const
{
    return *m_iter;
}

/** class CCAtomBondIter **/

template<>
void CCAtomBondIter::Init( CCAtom* parent )
{
	m_parent = parent;
	m_child = m_parent->BeginNbrBond( m_iter );
}

template<>
CCAtomBondIter CCAtomBondIter::operator++()
{
	m_child = m_parent->NextNbrBond( m_iter );
	return *this;
}

template<>
void* CCAtomBondIter::operator~() const
{
	return m_child->GetNbrAtom( m_parent );
}

/** class CCMolRingIter **/

template<>
void CCMolRingIter::Init( CCMol* parent )
{
	m_parent = parent;
    if ( !m_parent->HasFlag( MOL_SSSR_PERCEIVED ) ) {
        m_parent->BFSFindSSSR();
    }

    m_child = m_parent->BeginRing( m_iter );
}

template<>
CCMolRingIter CCMolRingIter::operator++()
{
    m_child = m_parent->NextRing( m_iter );
    return *this;
}

/** class CCMolChainIter **/

template<>
void CCMolChainIter::Init( CCMol* parent )
{
	m_parent = parent;
	m_child = m_parent->BeginChain( m_iter );
}

template<>
CCMolChainIter CCMolChainIter::operator++()
{
	m_child = m_parent->NextChain( m_iter );
	return *this;
}

/** class CCMolResidueIter **/

template<>
void CCMolResidueIter::Init( CCMol* parent )
{
    m_parent = parent;
    m_child = m_parent->BeginResidue( m_iter );
}

template<>
CCMolResidueIter CCMolResidueIter::operator++()
{
    m_child = m_parent->NextResidue( m_iter );
    return *this;
}

/** class CCChainResIter **/

template<>
void CCChainResIter::Init( CCChain* parent )
{
	m_parent = parent;
	m_child = m_parent->BeginResidue( m_iter );
}

template<>
CCChainResIter CCChainResIter::operator++()
{
	m_child = m_parent->NextResidue( m_iter );
	return ( *this );
}

/** class CCResidueAtomIter **/

template<>
void CCResidueAtomIter::Init( CCResidue* parent )
{
	m_parent = parent;
	m_child = m_parent->BeginAtom( m_iter );
}

template<>
CCResidueAtomIter CCResidueAtomIter::operator++()
{
	m_child = m_parent->NextAtom( m_iter );
	return ( *this );
}

/** class CCRingAtomIter **/

template<>
void CCRingAtomIter::Init(CCRing* parent)
{
	m_parent = parent;
	m_child = m_parent->BeginAtom( m_iter );
}

template<>
CCRingAtomIter CCRingAtomIter::operator++()
{
	m_child = m_parent->NextAtom( m_iter );
	return ( *this );
}

template<>
void* CCRingAtomIter::operator~() const
{
	vector<CCAtom*>::iterator m_j = m_iter;
	if ( m_parent->NextAtom( m_j ) == NULL ) {
		m_parent->BeginAtom( m_j );
	}
	return m_parent->GetMol()->GetBond( *m_iter, *m_j );
}

/** class CCMolAngleIter **/

CCMolAngleIter::CCMolAngleIter( CCMol* mol )
{
    // fill angle array
    CCAtom* atom1, * atom2, * atom3;
    vector<CCAtom*> angle;
    for ( int idx = 1; idx <= mol->GetNumAtom(); ++idx ) {
        atom2 = mol->GetAtom( idx );
        int numNbr = atom2->GetValence();
        if ( numNbr < 2 ) {
            continue;
        }

        for ( int i = 1; i < numNbr; ++i ) {
            atom1 = atom2->GetNbrAtom( i );
            for ( int j = i + 1; j <= numNbr; ++j ) {
                atom3 = atom2->GetNbrAtom( j );
                angle.clear();
                angle.push_back( atom1 );
                angle.push_back( atom2 );
                angle.push_back( atom3 );
                m_angles.push_back( angle );
            }
        }
    }

    m_mol = mol;
    m_i = ( m_angles.empty() ) ? m_angles.end() : m_angles.begin();
}

CCMolAngleIter::CCMolAngleIter( CCMol& mol )
{
    // fill angle array
    CCAtom* atom1, * atom2, * atom3;
    vector<CCAtom*> angle;
    for ( int idx = 1; idx <= mol.GetNumAtom(); ++idx ) {
        atom2 = mol.GetAtom( idx );
        int numNbr = atom2->GetValence();
        if ( numNbr < 2 ) {
            continue;
        }

        for ( int i = 1; i < numNbr; ++i ) {
            atom1 = atom2->GetNbrAtom( i );
            for ( int j = i + 1; j <= numNbr; ++j ) {
                atom3 = atom2->GetNbrAtom( j );
                angle.clear();
                angle.push_back( atom1 );
                angle.push_back( atom2 );
                angle.push_back( atom3 );
                m_angles.push_back( angle );
            }
        }
    }

    m_mol = &mol;
    m_i = ( m_angles.empty() ) ? m_angles.end() : m_angles.begin();
}

CCMolAngleIter::CCMolAngleIter( const CCMolAngleIter& ai )
{
    m_i = ai.m_i;
    m_mol = ai.m_mol;
    m_angles = ai.m_angles;
}

CCMolAngleIter& CCMolAngleIter::operator=( const CCMolAngleIter& ai )
{
    if ( this != &ai ) {
        m_i = ai.m_i;
        m_mol = ai.m_mol;
        m_angles = ai.m_angles;
    }
    return *this;
}

CCMolAngleIter& CCMolAngleIter::operator++()
{
    ++m_i;
    return *this;
}

/** class CCMolTorsionIter **/

CCMolTorsionIter::CCMolTorsionIter( CCMol* mol )
{
    // fill torsion array
    CCBond* bond;
    CCAtom* atom1, * atom2, * atom3, * atom4;
    vector<CCAtom*> torsion;
    for ( int idx = 1; idx <= mol->GetNumBond(); ++idx ) {
        bond = mol->GetBond( idx );
        atom2 = bond->GetBeginAtom();
        atom3 = bond->GetEndAtom();

        int numNbr2 = atom2->GetValence();
        int numNbr3 = atom3->GetValence();

        if ( numNbr2 < 2 || numNbr3 < 2 ) {
            continue;
        }

        for ( int i = 1; i <= numNbr2; ++i ) {
            atom1 = atom2->GetNbrAtom( i );
            if ( atom1 == atom3 ) {
                continue;
            }

            for ( int j = 1; j <= numNbr3; ++j ) {
                atom4 = atom3->GetNbrAtom( j );
                if ( atom4 == atom2 ) {
                    continue;
                }
                torsion.clear();
                torsion.push_back( atom1 );
                torsion.push_back( atom2 );
                torsion.push_back( atom3 );
                torsion.push_back( atom4 );
                m_torsions.push_back( torsion );
            }
        }
    }

    m_mol = mol;
    m_i = ( m_torsions.empty() ) ? m_torsions.end() : m_torsions.begin();
}

CCMolTorsionIter::CCMolTorsionIter( CCMol& mol )
{
    // fill torsion array
    CCBond* bond;
    CCAtom* atom1, * atom2, * atom3, * atom4;
    vector<CCAtom*> torsion;
    for ( int idx = 1; idx <= mol.GetNumBond(); ++idx ) {
        bond = mol.GetBond( idx );
        atom2 = bond->GetBeginAtom();
        atom3 = bond->GetEndAtom();

        int numNbr2 = atom2->GetValence();
        int numNbr3 = atom3->GetValence();

        if ( numNbr2 < 2 || numNbr3 < 2 ) {
            continue;
        }

        for ( int i = 1; i <= numNbr2; ++i ) {
            atom1 = atom2->GetNbrAtom( i );
            if ( atom1 == atom3 ) {
                continue;
            }

            for ( int j = 1; j <= numNbr3; ++j ) {
                atom4 = atom3->GetNbrAtom( j );
                if ( atom4 == atom2 ) {
                    continue;
                }
                torsion.clear();
                torsion.push_back( atom1 );
                torsion.push_back( atom2 );
                torsion.push_back( atom3 );
                torsion.push_back( atom4 );
                m_torsions.push_back( torsion );
            }
        }
    }

    m_mol = &mol;
    m_i = ( m_torsions.empty() ) ? m_torsions.end() : m_torsions.begin();
}

CCMolTorsionIter::CCMolTorsionIter( const CCMolTorsionIter& ti )
{
    m_i = ti.m_i;
    m_mol = ti.m_mol;
    m_torsions = ti.m_torsions;
}

CCMolTorsionIter& CCMolTorsionIter::operator=( const CCMolTorsionIter& ti )
{
    if ( this != &ti ) {
        m_i = ti.m_i;
        m_mol = ti.m_mol;
        m_torsions = ti.m_torsions;
    }
    return *this;
}

CCMolTorsionIter& CCMolTorsionIter::operator++()
{
    ++m_i;
    return *this;
}

/** class CCMolPairIter **/

CCMolPairIter::CCMolPairIter( CCMol* mol )
{
    CCAtom* atom1, * atom2, * atom4;
    vector<CCAtom*> apair;
    bool notpair;
    for ( int idx1 = 1; idx1 < mol->GetNumAtom(); ++idx1 ) {
        atom1 = mol->GetAtom( idx1 );
        for ( int idx4 = idx1 + 1; idx4 <= mol->GetNumAtom(); ++idx4 ) {
            notpair = false;
            atom4 = mol->GetAtom( idx4 );
            if ( atom1->IsConnected( atom4 ) ) {
                // 1-2 connected
                notpair = true;
            }

            for ( int i = 1; i <= atom1->GetValence(); ++i ) {
                if ( notpair ) {
                    break;
                }

                atom2 = atom1->GetNbrAtom( i );
                if ( atom4->IsConnected( atom2 ) ) {
                    // 1-3 connected
                    notpair = true;
                }
            }

            if ( !notpair ) {
                apair.clear();
                apair.push_back( atom1 );
                apair.push_back( atom4 );
                m_pairs.push_back( apair );
            }
        }
    }

    m_mol = mol;
    m_i = ( m_pairs.empty() ) ? m_pairs.end() : m_pairs.begin();
}

CCMolPairIter::CCMolPairIter( CCMol& mol )
{
    CCAtom* atom1, * atom2, * atom4;
    vector<CCAtom*> apair;
    bool notpair;
    for ( int idx1 = 1; idx1 < mol.GetNumAtom(); ++idx1 ) {
        atom1 = mol.GetAtom( idx1 );
        for ( int idx4 = idx1 + 1; idx4 <= mol.GetNumAtom(); ++idx4 ) {
            notpair = false;
            atom4 = mol.GetAtom( idx4 );
            if ( atom1->IsConnected( atom4 ) ) {
                // 1-2 connected
                notpair = true;
            }

            for ( int i = 1; i <= atom1->GetValence(); ++i ) {
                if ( notpair ) {
                    break;
                }

                atom2 = atom1->GetNbrAtom( i );
                if ( atom4->IsConnected( atom2 ) ) {
                    // 1-3 connected
                    notpair = true;
                }
            }

            if ( !notpair ) {
                apair.clear();
                apair.push_back( atom1 );
                apair.push_back( atom4 );
                m_pairs.push_back( apair );
            }
        }
    }

    m_mol = &mol;
    m_i = ( m_pairs.empty() ) ? m_pairs.end() : m_pairs.begin();
}

CCMolPairIter::CCMolPairIter( const CCMolPairIter& pi )
{
    m_i = pi.m_i;
    m_mol = pi.m_mol;
    m_pairs = pi.m_pairs;
}

CCMolPairIter& CCMolPairIter::operator=( const CCMolPairIter& pi )
{
    if ( this != &pi ) {
        m_i = pi.m_i;
        m_mol = pi.m_mol;
        m_pairs = pi.m_pairs;
    }
    return *this;
}

CCMolPairIter& CCMolPairIter::operator++()
{
    ++m_i;
    return *this;
}

/** class CCMol14PairIter **/

CCMol14PairIter::CCMol14PairIter( CCMol* mol )
{
    CCAtom* atom1, * atom2, * atom3, * atom4;
    vector<CCAtom*> apair;
    bool is14;

    for ( int idx1 = 1; idx1 < mol->GetNumAtom(); ++idx1 ) {
        atom1 = mol->GetAtom( idx1 );
        for ( int idx2 = idx1 + 1; idx2 <= mol->GetNumAtom(); ++idx2 ) {
            atom4 = mol->GetAtom( idx2 );
            if ( atom1->IsConnected( atom4 ) ) {
                // 1-2 connected
                continue;
            }
            is14 = false;

            for ( int i = 1; !is14 && i <= atom1->GetValence(); ++i ) {
                atom2 = atom1->GetNbrAtom( i );
                for ( int j = 1; !is14 && j <= atom4->GetValence(); ++j ) {
                    atom3 = atom4->GetNbrAtom( j );
                    if ( atom2->IsConnected( atom3 ) ) {
                        is14 = true;
                    }
                }
            }

            if ( is14 ) {
                apair.clear();
                apair.push_back( atom1 );
                apair.push_back( atom4 );
                m_pairs.push_back( apair );
            }
        }
    }

    m_mol = mol;
    m_i = ( m_pairs.empty() ) ? m_pairs.end() : m_pairs.begin();
}

CCMol14PairIter::CCMol14PairIter( CCMol& mol )
{
    CCAtom* atom1, * atom2, * atom3, * atom4;
    vector<CCAtom*> apair;
    bool is14;

    for ( int idx1 = 1; idx1 < mol.GetNumAtom(); ++idx1 ) {
        atom1 = mol.GetAtom( idx1 );
        for ( int idx2 = idx1 + 1; idx2 <= mol.GetNumAtom(); ++idx2 ) {
            atom4 = mol.GetAtom( idx2 );
            if ( atom1->IsConnected( atom4 ) ) {
                // 1-2 connected
                continue;
            }
            is14 = false;

            for ( int i = 1; !is14 && i <= atom1->GetValence(); ++i ) {
                atom2 = atom1->GetNbrAtom( i );
                for ( int j = 1; !is14 && j <= atom4->GetValence(); ++j ) {
                    atom3 = atom4->GetNbrAtom( j );
                    if ( atom2->IsConnected( atom3 ) ) {
                        is14 = true;
                    }
                }
            }

            if ( is14 ) {
                apair.clear();
                apair.push_back( atom1 );
                apair.push_back( atom4 );
                m_pairs.push_back( apair );
            }
        }
    }

    m_mol = &mol;
    m_i = ( m_pairs.empty() ) ? m_pairs.end() : m_pairs.begin();
}

CCMol14PairIter::CCMol14PairIter( const CCMol14PairIter& pi )
{
    m_i = pi.m_i;
    m_mol = pi.m_mol;
    m_pairs = pi.m_pairs;
}

CCMol14PairIter& CCMol14PairIter::operator=( const CCMol14PairIter& pi )
{
    if ( this != &pi ) {
        m_i = pi.m_i;
        m_mol = pi.m_mol;
        m_pairs = pi.m_pairs;
    }
    return *this;
}

CCMol14PairIter& CCMol14PairIter::operator++()
{
    ++m_i;
    return *this;
}

/** class CCMol1NPairIter **/

CCMol1NPairIter::CCMol1NPairIter( CCMol* mol )
{
    CCAtom* atom1, * atom2, * atom3, * atom4;
    vector<CCAtom*> apair;
    bool not14;
    for ( int idx1 = 1; idx1 < mol->GetNumAtom(); ++idx1 ) {
        atom1 = mol->GetAtom( idx1 );
        for ( int idx2 = idx1 + 1; idx2 <= mol->GetNumAtom(); ++idx2 ) {
            atom2 = mol->GetAtom( idx2 );
            not14 = false;

            for ( int i = 1; !not14 && i <= atom1->GetValence(); ++i ) {
                atom3 = atom1->GetNbrAtom( i );
                if ( atom3 == atom2 ) {
                    // 1-2 connected
                    not14 = true;
                }
                for ( int j = 1; !not14 && j <= atom3->GetValence(); ++j ) {
                    atom4 = atom3->GetNbrAtom( j );
                    if ( atom4 == atom2 ) {
                        // 1-3 connected
                        not14 = true;
                    }
                    for ( int k = 1;
                          !not14 && k <= atom4->GetValence();
                          ++k ) {
                        if ( atom4->GetNbrAtom( k ) == atom2 ) {
                            // 1-4 connected
                            not14 = true;
                        }
                    }
                }
            }

            if ( !not14 ) {
                apair.clear();
                apair.push_back( atom1 );
                apair.push_back( atom2 );
                m_pairs.push_back( apair );
            }
        }
    }

    m_mol = mol;
    m_i = ( m_pairs.empty() ) ? m_pairs.end() : m_pairs.begin();
}

CCMol1NPairIter::CCMol1NPairIter( CCMol& mol )
{
    CCAtom* atom1, * atom2, * atom3, * atom4;
    vector<CCAtom*> apair;
    bool not14;
    for ( int idx1 = 1; idx1 < mol.GetNumAtom(); ++idx1 ) {
        atom1 = mol.GetAtom( idx1 );
        for ( int idx2 = idx1 + 1; idx2 <= mol.GetNumAtom(); ++idx2 ) {
            atom2 = mol.GetAtom( idx2 );
            not14 = false;

            for ( int i = 1; !not14 && i <= atom1->GetValence(); ++i ) {
                atom3 = atom1->GetNbrAtom( i );
                if ( atom3 == atom2 ) {
                    // 1-2 connected
                    not14 = true;
                }
                for ( int j = 1; !not14 && j <= atom3->GetValence(); ++j ) {
                    atom4 = atom3->GetNbrAtom( j );
                    if ( atom4 == atom2 ) {
                        // 1-3 connected
                        not14 = true;
                    }
                    for ( int k = 1;
                          !not14 && k <= atom4->GetValence();
                          ++k ) {
                        if ( atom4->GetNbrAtom( k ) == atom2 ) {
                            // 1-4 connected
                            not14 = true;
                        }
                    }
                }
            }

            if ( !not14 ) {
                apair.clear();
                apair.push_back( atom1 );
                apair.push_back( atom2 );
                m_pairs.push_back( apair );
            }
        }
    }

    m_mol = &mol;
    m_i = ( m_pairs.empty() ) ? m_pairs.end() : m_pairs.begin();
}

CCMol1NPairIter::CCMol1NPairIter( const CCMol1NPairIter& pi )
{
    m_i = pi.m_i;
    m_mol = pi.m_mol;
    m_pairs = pi.m_pairs;
}

CCMol1NPairIter& CCMol1NPairIter::operator=( const CCMol1NPairIter& pi )
{
    if ( this != &pi ) {
        m_i = pi.m_i;
        m_mol = pi.m_mol;
        m_pairs = pi.m_pairs;
    }
    return *this;
}

CCMol1NPairIter& CCMol1NPairIter::operator++()
{
    ++m_i;
    return *this;
}
