#include <ccbg/CCRing.h>
#include <ccbg/CCMol.h>
#include <iterator>

void ConstructAllRingsfromAssembly( vector<CCRing*>& allring,
                                    vector<CCRing*>& ringAssembly,
                                    int sizeLimit )
{
    // all rings will be expanded into ringAssembly
    // initialization
    CCRingIter i;
    for ( i = ringAssembly.begin(); i != ringAssembly.end(); ++i ) {
        CCRing* ring = new CCRing( **i );
        allring.push_back( ring );
    }

    int NBRofSSSR = allring.size();

    if ( NBRofSSSR <= 1 ) {
        return;
    }
    CCMol* mol = allring[0]->GetMol();

    // initialization
    vector<CCBitVec> bvs( 1 ); // bvs[0] is a token
    for ( int k = 1; k <= NBRofSSSR; ++k ) {
        CCBitVec b( NBRofSSSR + 1 );
        b.Set( k );
        bvs.push_back( b );
    }

    // begin alogrithm
    int j = 1/*, count=NBRofSSSR*/;
    int p;
    CCBitVec b1, b2, newbv;
    for ( ; j < allring.size(); ++j ) {
        if ( j <= NBRofSSSR ) {
            p = j + 1;
        }
        else {
            p = 1;
        }

        // begin combination
        for ( ; p <= NBRofSSSR; ++p ) {
            if ( ( bvs[p] & bvs[j] ).None() ) {
                b1 = allring[j - 1]->GetBVPath();
                b2 = allring[p - 1]->GetBVPath();
                if ( !( b1 & b2 ).None() ) {
                    newbv = b1 ^ b2;
                    if ( !AnExistRing( newbv, allring )
                      && ( sizeLimit == -1 || newbv.Count() <= sizeLimit ) ) {
                        // save this ring
                        CCRing* ring = new CCRing; AssertMemAlloc( ring );
                        ring->SetMol( mol );
                        ring->SetAtomsFromBVPath( newbv );
                        ring->SetBVPath( newbv );
                        allring.push_back( ring );
                        // upadate
                        bvs.push_back( bvs[j] | bvs[p] );
                        /*if (++count%1000==0)
                                        cout << count << "\n";*/
                    }
                }
            }
        }
    }
}

bool AnExistRing( CCBitVec& newRingBV, vector<CCRing*>& curRings )
{
    for ( vector<CCRing*>::const_iterator i = curRings.begin();
          i != curRings.end();
          ++i ) {
        CCBitVec curBV = ( *i )->GetBVPath();
        if ( ( newRingBV == curBV )
          || ( ( ( newRingBV & curBV ) == curBV )
            || ( ( newRingBV & curBV ) == newRingBV ) ) ) // linear dependent
        {
            return true;
        }
    }

    return false;
}

void AssembleRings( const vector<CCRing*>& sssr,
                    vector<vector<CCRing*> >& ringAssemblys )
{
    if ( sssr.empty() ) {
        return;
    }
    if ( sssr.size() == 1 ) {
        ringAssemblys.push_back( sssr );
        return;
    }

    list<CCRing*> remainRings;
    list<CCRing*>::iterator i;
    copy( sssr.begin(), sssr.end(), back_inserter( remainRings ) );

    // initialization
    vector<CCRing*> tmpRingAssembly;
    bool turnFINISH;
    CCBitVec bvpath0 = remainRings.front()-> GetBVPath(), bvpath;

    do {
        turnFINISH = true;

        for ( i = remainRings.begin(); i != remainRings.end(); ) {
            bvpath = ( *i )->GetBVPath();
            if ( !( bvpath0 & bvpath ).None() ) {
                turnFINISH = false;
                bvpath0 |= bvpath;
                tmpRingAssembly.push_back( *i );
                i = remainRings.erase( i );
            }
            else {
                ++i;
            }
        }

        if ( remainRings.empty() ) {
            ringAssemblys.push_back( tmpRingAssembly );
            break;
        }

        if ( turnFINISH ) // and remainRings is not empty
        {
            ringAssemblys.push_back( tmpRingAssembly );
            // update
            tmpRingAssembly.clear();
            bvpath0 = remainRings.front()->GetBVPath();
        }
    }
    while ( !remainRings.empty() );
}

bool BuildRingTables( CCMol& mol, vector<CCRingTable*>& rtables )
{
    CCBitVec usedAtoms( mol.GetNumAtom() + 1 );
    CCRingNode* rnode( NULL ),* nbrnode( NULL );

    FOR_ATOMS_OF_MOL( rootAtom, mol ) {
        if ( !usedAtoms[rootAtom->GetIdx()] && rootAtom->IsInRing() ) {
            CCRingTable* rt = new CCRingTable; AssertMemAlloc( rt );

            rt->SetMol( &mol );

            // initialization
            queue<CCRingNode*> q;
            rnode = new CCRingNode; AssertMemAlloc( rnode );

            rnode->SetAtom( &*rootAtom );
            rt->AddNode( rnode );
            usedAtoms.Set( rnode->GetIdx() );
            q.push( rnode );

            // begin algorithm
            do {
                rnode = q.front();
                q.pop();

                FOR_NBORS_OF_ATOM( nbrAtom, rnode->GetAtom() ) {
                    if ( ( (CCBond*)~nbrAtom )->IsInRing() ) // nbrBond in ring
                    {
                        if ( !usedAtoms[nbrAtom->GetIdx()] ) {
                            nbrnode = new CCRingNode;AssertMemAlloc( nbrnode );

                            nbrnode->SetAtom( &*nbrAtom );
                            rt->AddNode( nbrnode );
                            usedAtoms.Set( nbrAtom->GetIdx() );
                            q.push( nbrnode );
                        }
                        else {
                            nbrnode = rt->GetNode( &*nbrAtom );
                        }

                        // update connection
                        rnode->AddNbr( nbrnode );
                    }
                }
            }
            while ( !q.empty() );

            // save ring table
            rtables.push_back( rt );
        }
    }

    return !rtables.empty();
}
//////////////////////////////////////////////////////////////////////////
CCRing::CCRing( const CCRing& src )
    : m_mol( src.m_mol ), m_vatom( src.m_vatom ), m_bvpath( src.m_bvpath )
{
    CCBase::Clone( src );
}

CCRing& CCRing::operator=( const CCRing& src )
{
    if ( this != &src ) {
        CCBase::Clone( src );
        m_mol = src.m_mol;
        m_vatom = src.m_vatom;
        m_bvpath = src.m_bvpath;
    }
    return *this;
}
CCAtom* CCRing::BeginAtom( vector<CCAtom*>::iterator& i )
{
    i = m_vatom.begin();
    return ( i == m_vatom.end() ? NULL : *i );
}

CCAtom* CCRing::NextAtom( vector<CCAtom*>::iterator& i )
{
    i++;
    return ( i == m_vatom.end() ? NULL : *i );
}

vector<int> CCRing::GetAtomPath() const
{
    vector<int> vi;
    FOR_ATOMS_OF_RING( atom, ( CCRing * )this ) {
        vi.push_back( atom->GetIdx() );
    }
    return vi;
}

CCBitVec CCRing::GetAVPath() const
{
    CCBitVec avpath;
    avpath.Resize( m_mol->GetNumAtom() + 1 );
    FOR_ATOMS_OF_RING( atom, ( CCRing * ) this ) {
        avpath.Set( atom->GetIdx() );
    }

    return avpath;
}

vector<int> CCRing::GetBondPath() const
{
    int ringsize = m_vatom.size();
    vector<int> bpath;
    for ( int i = 0; i < ringsize; ++i ) {
        bpath.push_back( m_mol->GetBond( m_vatom[i],
                                         m_vatom[( i + 1 ) % ringsize] )->GetIdx() );
    }

    return bpath;
}

vector<CCBond*> CCRing::GetBonds() const
{
    int ringsize = m_vatom.size();
    vector<CCBond*> vbond;
    for ( int i = 0; i < ringsize; ++i ) {
        vbond.push_back( m_mol->GetBond( m_vatom[i],
                                         m_vatom[( i + 1 ) % ringsize] ) );
    }

    return vbond;
}

void CCRing::SetAtoms( const vector<int>& apath )
{
    m_vatom.clear();
    CCAtom* atom( NULL );
    for ( int i = 0; i < apath.size(); ++i ) {
        if ( atom = m_mol->GetAtom( apath[i] ) ) {
            m_vatom.push_back( atom );
        }
    }
}

void CCRing::WriteRingAtoms( ostream& os/*=cout*/ ) const
{
    vector<int> apath = GetAtomPath();
    copy( apath.begin(), apath.end(), std::ostream_iterator<int>( os, " " ) ); 
    os << "\n";
}

void CCRing::WriteRingBonds( ostream& os/*=cout*/ ) const
{
    vector<int> bpath = GetBondPath();
    copy( bpath.begin(), bpath.end(), std::ostream_iterator<int>( os, " " ) ); 
    cout << "\n";
}

bool CCRing::IsAtomInRing( CCAtom* atom ) const
{
    return IsAtomInRing( atom->GetIdx() );
}

bool CCRing::IsAtomInRing( int idx ) const
{
    FOR_ATOMS_OF_RING( atom, ( CCRing * ) this ) {
        if ( atom->GetIdx() == idx ) {
            return true;
        }
    }
    return false;
}

bool CCRing::IsBondInRing( CCBond* bond ) const
{
    return ( bond && m_bvpath[bond->GetIdx()] );
}

void CCRing::SetAtomsFromBVPath( const CCBitVec& bvpath )
{
    CCMol*& mol = m_mol;
    CCAtom* atom = NULL;

    // start from first bond
    for ( int i = 1; i < bvpath.Bits(); ++i ) {
        if ( bvpath[i] ) {
            atom = mol->GetBond( i )->GetBeginAtom();
            break;
        }
    }

    // begin algorithm
    bool FINISH = false;
    CCBondIter i;
    CCBond* bond = NULL;
    int bId;
    CCBitVec used( bvpath.Bits() );

    do {
        FINISH = true;
        for ( bond = atom->BeginNbrBond( i );
              bond;
              bond = atom->NextNbrBond( i ) ) {
            bId = bond->GetIdx();
            if ( bvpath[bId] && !used[bId] ) {
                FINISH = false;
                used.Set( bId );
                atom = bond->GetNbrAtom( atom );
                // set data
                m_vatom.push_back( atom );
                break;
            }
        }
    }
    while ( !FINISH );
}

bool CCRing::IsAromatic()
{
    if ( HasFlag( RING_IS_AROMATIC ) ) {
        return true;
    }

    int i, ringsize = GetSize();
	CCBond *pbond(NULL);
    for ( i = 0; i < ringsize; ++i ) {
		pbond = m_mol->GetBond( m_vatom[i], m_vatom[( i + 1 ) % ringsize] );
		if ( !pbond ) {
			// should never go to here
			string msg = "Can not get ring bond.\n";
			msg += this->GetMol()->GetName() + "\n";
			gErrorLog.Throw( __FUNCTION__, msg, ccWarning );
			return false;	              
		}
        if ( pbond->IsAromatic() ) {
            continue;
        }
		return false;
    }

    SetFlag( RING_IS_AROMATIC );
    return true;
}

// five or less membered rings with the average in-ring torsion 
// less than 7.5 degrees will be considered as planar ring. The 
// threshold is 12.0 degree for six or more membered rings.
bool CCRing::IsNearPlanar()
{
    static double FLATRINGFIVE = 7.5;
    static double FLATRINGSIX = 12.0;

    double torsions( 0.0 );
    int rsize = m_vatom.size();
    if ( rsize == 3 ) {
        return true;
    }

    for ( int i = 0; i < rsize; ++i ) {
        double torsion = Torsion( m_vatom[i]->GetXYZ(),
                                  m_vatom[( i + 1 ) % rsize]->GetXYZ(),
                                  m_vatom[( i + 2 ) % rsize]->GetXYZ(),
                                  m_vatom[( i + 3 ) % rsize]->GetXYZ() );
        torsions += MinTorsion( torsion );
    }

    torsions = torsions / rsize;
    if ( ( rsize <= 5 && torsions < FLATRINGFIVE )
      || ( rsize > 5 && torsions < FLATRINGSIX ) ) {
        return true;
    }
    return false;
}
void CCMol::PerceiveFragment()
{
    if ( HasFlag( MOL_FRAGMENT_PERCEIVED ) ) {
        return;
    }
    SetFlag( MOL_FRAGMENT_PERCEIVED );

    int numAtoms = GetNumAtom();
    CCBitVec used( numAtoms + 1 );

    int head, part = 0;

    // in most cases, there is only one fragment
    for ( int i = 1; i <= numAtoms && used.Count() != numAtoms; ++i ) {
        if ( !used[i] ) {
            queue<int> q;
            CCAtom* atom;

            q.push( i );
            used.Set( i );

            /*vector<int> frag;
                    frag.push_back(i);*/

            while ( !q.empty() ) {
                head = q.front();
                q.pop();
                //  frag.push_back(head->GetIdx());

                atom = GetAtom( head );
                atom->SetPart( part + 1 );

				FOR_NBORS_OF_ATOM( nbr, atom ) {
					if ( !used[nbr->GetIdx()] ) {
						q.push( nbr->GetIdx() );
						used.Set( nbr->GetIdx() );
					}
				}
            }

            part++;
            //  cfl.push_back(frag);
        }
    }

    SetNumSubs( part );
}

void CCRing::UpdateBVPath()
{
    int ringsize = m_vatom.size();
	m_bvpath.Clear();
	m_bvpath.Resize( m_mol->GetNumBond() + 1 );
    CCBond* bond( NULL );
    for ( int i = 0; i < ringsize; ++i ) {
        bond = m_mol->GetBond( m_vatom[i], m_vatom[( i + 1 ) % ringsize] );
        if ( bond ) {
            m_bvpath.Set( bond->GetIdx() );
        }
    }
}
void FindRings( CCMol& mol, vector<int>& path, CCBitVec& avisit,
                CCBitVec& bvisit, int root, int depth )
{
    CCAtom* atom;
    CCBond* bond;
    CCBondIter k;

    if ( avisit[root] ) // find a ring
    {
        int j = depth - 1;
        bond = mol.GetBond( path[j--] );
        bond->SetInRing();
        while ( j >= 0 ) {
            bond = mol.GetBond( path[j--] );
            bond->SetInRing();
            ( bond->GetBeginAtom() )->SetInRing();
            ( bond->GetEndAtom() )->SetInRing();
            if ( bond->GetBeginAtomIdx() == root
              || bond->GetEndAtomIdx() == root ) {
                break;
            }
        }
    }
    else {
        avisit.Set( root ); // set atom visited
        atom = mol.GetAtom( root );
        for ( bond = atom->BeginNbrBond( k );
              bond;
              bond = atom->NextNbrBond( k ) ) {
            if ( !bvisit[bond->GetIdx()] ) {
                path[depth] = bond->GetIdx();
                bvisit.Set( bond->GetIdx() ); // set bond visited
                FindRings( mol, path, avisit, bvisit,
                           bond->GetNbrAtom(atom)->GetIdx(), depth + 1 );
            }
        }
    }
}

void CCMol::PerceiveRingAtomsAndBonds()
{
    if ( HasFlag( MOL_RING_PERCEIVED ) ) {
        return;
    }
    SetFlag( MOL_RING_PERCEIVED );

    CCBitVec avisit, bvisit;
    avisit.Resize( GetNumAtom() + 1 );
    bvisit.Resize( GetNumBond() + 1 );
    vector<int> path;
    path.resize( GetNumBond() );

    for ( int i = 1; i <= GetNumAtom(); ++i ) {
        if ( !avisit[i] ) {
            FindRings( *this, path, avisit, bvisit, i, 0 );
        }
    }
}

void CCMol::FindAllRingsfromSSSR( vector<CCRing*>& allRings,
                                  int sizeLimit/* =22 */ )
{
    vector<CCRing*>& sssr = GetSSSR();
    vector<vector<CCRing*> > ringAssemblys;
    AssembleRings( sssr, ringAssemblys );

    for ( vector<vector<CCRing*> >::iterator i = ringAssemblys.begin();
          i != ringAssemblys.end();
          ++i ) {
        ConstructAllRingsfromAssembly( allRings, *i, sizeLimit );
    }
}

void CCMol::BFSFindSSSR()
{
    if ( HasFlag( MOL_SSSR_PERCEIVED ) ) {
        return;
    }
    SetFlag( MOL_SSSR_PERCEIVED );

	// begin SSSR detection
	m_vring.clear();

    int frj = GetNumBond() - GetNumAtom() + GetNumSubs();
    if ( frj <= 0 ) {
        return;
    } // no ring in molecule

    PerceiveRingAtomsAndBonds();

    // build ring connection table base on molecular connection table
    vector<CCRingTable*> rtables;
    if ( !BuildRingTables( *this, rtables ) ) {
        return;
    } // rtable is empty, (should never be)

    vector<CCRingTable*>::iterator i = rtables.begin();
    for ( ; i != rtables.end(); ++i ) {
        ( *i )->BFSFindSSSR();
        vector<CCRing*>& vring = ( *i )->GetRings();
        copy( vring.begin(), vring.end(), back_inserter( m_vring ) );
        // release memory
        delete * i;
    }

    if ( GetSSSR().size() != frj ) {
        gErrorLog.Throw( __FUNCTION__,
                         "Something must be wrong when perceiving SSSR!",
                         ccWarning );
    }
}

void CCMol::DFSFindAllRings( int sizeLimit )
{
    if ( HasFlag( MOL_ALL_RING_PERCEIVED ) ) {
        return;
    }
    SetFlag( MOL_ALL_RING_PERCEIVED );

	m_vring.clear();

    // build ring connection table base on molecular connection table
    vector<CCRingTable*> rtables;
    if ( !BuildRingTables( *this, rtables ) ) {
        return;
    } // rtable is empty

    vector<CCRingTable*>::iterator rt = rtables.begin();
    for ( ; rt != rtables.end(); ++rt ) {
#if 1
        // go deep down to 14 atoms (default), enough for aromaticity detection
        for ( CCRingNode*rnode = ( *rt )->BeginNode();
              rnode;
              rnode = ( *rt )->NextNode( rnode ) ) {
            // detect from closure bond
            if ( rnode->GetAtom()->IsInClosureBond() ) {
                ( *rt )->DFSRing( rnode, m_vring, sizeLimit );

                // all rings should have been found out here, always true 
                if ( ( *rt )->GetSize() <= sizeLimit ) {
                    break;
                }
            }
        }
#else
        // no limitation, find all ring
        ( *rt )->DFSRing( ( *rt )->BeginNode(), m_vring, -1 );
#endif
    }
}

//////////////////////////////////////////////////////////////////////////
CCRingNode::CCRingNode( const CCRingNode& rd )
    : m_atom( rd.m_atom ), m_degree( rd.m_degree ), m_nbrs( rd.m_nbrs )
{
}

CCRingNode& CCRingNode::operator=( const CCRingNode& rd )
{
    m_atom = rd.m_atom;
    m_degree = rd.m_degree;
    m_nbrs = rd.m_nbrs;
    return *this;
}

int CCRingNode::GetIdx() const
{
    return ( m_atom ? m_atom->GetIdx() : 0 );
}

CCRingNode* CCRingNode::BeginNbr()
{
    return m_nbrs.empty() ? NULL : m_nbrs[0];
}

CCRingNode* CCRingNode::NextNbr( CCRingNode* last )
{
    if ( !last ) {
        gErrorLog.Throw( __FUNCTION__, "Last neighbor is NULL!", ccWarning );
        return NULL;
    }

    vector<CCRingNode*>::iterator i;
    for ( i = m_nbrs.begin(); i != m_nbrs.end(); ++i ) {
        if ( *i == last ) {
            ++i;
            break;
        }
    }

    return i != m_nbrs.end() ? ( *i ) : NULL;
}

void CCRingNode::AddNbr( CCRingNode* nbr ) // Add or restore a neighbor
{
    vector<CCRingNode*>::iterator i;
    for ( i = m_nbrs.begin(); i != m_nbrs.end(); ++i ) {
        if ( *i == nbr ) {
            return;
        }
    }       // do nth with an existing nbr

    m_nbrs.push_back( nbr );
    m_degree++;
}

CCRingNode* CCRingNode::RemoveNbr( CCRingNode* nbr )
{
    vector<CCRingNode*>::iterator i;
    for ( i = m_nbrs.begin(); i != m_nbrs.end(); ++i ) {
        if ( *i == nbr ) {
            i = m_nbrs.erase( i );
            m_degree--;
            break;
        }
    }

    return i != m_nbrs.end() ? ( *i ) : NULL;
}

//////////////////////////////////////////////////////////////////////////
CCRingTable::CCRingTable( const CCRingTable& rt )
    : m_mol( rt.m_mol ), m_rt( rt.m_rt )
{
}

CCRingTable& CCRingTable::operator=( const CCRingTable& rt )
{
    m_mol = rt.m_mol;
    m_rt = rt.m_rt;
    return *this;
}

CCRingTable::~CCRingTable()
{
    vector<CCRingNode*>::iterator i = m_rt.begin();
    for ( ; i != m_rt.end(); ++i ) {
        if ( *i ) {
            delete * i;
            *i = NULL;
        }
    }
    // do not free m_vring
}

void CCRingTable::DFSRing( CCRingNode* rootnode, vector<CCRing*>& allRings,
                           int sizeLimit )
{
    if ( allRings.size() > 10000 ) {
        gErrorLog.Throw( __FUNCTION__, "Too many ring detected!", ccWarning );
        return;
    }

    CCMol* mol = GetMol();

    // we can save two variables by using 'static' declaration :-)
    // both will be restored to initial states when DFS finished
    static vector<int> apath;
    static CCBitVec visit;
	visit.Resize( mol->GetNumAtom() + 1 );

    int prevNodeIdx = -1;
    if ( !apath.empty() ) {
        prevNodeIdx = apath.back();
    }

    int rootIdx = rootnode->GetIdx();
    visit.Set( rootIdx );
    apath.push_back( rootIdx );

    if ( sizeLimit == 0 ) {
        // can not go deeper 
        goto BACK;
    }

    for ( CCRingNode*nbrnode = rootnode->BeginNbr();
          nbrnode;
          nbrnode = rootnode->NextNbr( nbrnode ) ) {
        int nbrIdx = nbrnode->GetIdx();
        if ( nbrIdx == prevNodeIdx ) {
            continue;
        }
        else if ( visit[nbrIdx] ) // found a ring
        {
            vector<int> ringPath;
            vector<int>::iterator pos;
            pos = std::find( apath.begin(), apath.end(), nbrIdx );
            for ( pos; pos != apath.end(); ++pos ) {
                ringPath.push_back( *pos );
            }

            int ringsize = ringPath.size();

            CCBitVec bvpath( mol->GetNumBond() + 1 );
            for ( int j = 0; j < ringsize; ++j ) {
                bvpath.Set( mol->GetBond( ringPath[j],
                                          ringPath[( j + 1 ) % ringsize] )->GetIdx() );
            }

            // a new ring
            if ( !AnExistRing( bvpath, allRings ) ) {
                CCRing* ring = new CCRing; AssertMemAlloc( ring );

                ring->SetMol( mol );
                // ring->SetAtomPath( ringPath );
                ring->SetAtoms( ringPath );
                ring->SetBVPath( bvpath );

                // sort rings by its size
                CCRingIter k;
                for ( k = allRings.begin(); k != allRings.end(); ++k ) {
                    if ( ringsize > ( *k )->GetSize() ) {
                        break;
                    }
                }
                allRings.insert( k, ring );
            }
        }
        else {
            DFSRing( nbrnode, allRings, sizeLimit - 1 );
        }
    }

    BACK : apath.pop_back();
    visit.Reset( rootIdx );
}

CCRingNode* CCRingTable::GetNode( CCAtom* atom ) const
{
    for ( int i = 0; i < m_rt.size(); ++i ) {
        if ( m_rt[i]->GetAtom() == atom ) {
            return m_rt[i];
        }
    }
    return NULL;
}

CCRingNode* CCRingTable::BeginNode()
{
    return m_rt.empty() ? NULL : m_rt[0];
}

CCRingNode* CCRingTable::NextNode( CCRingNode* last )
{
    if ( !last ) {
        gErrorLog.Throw( __FUNCTION__, "Last ring node is NULL!", ccWarning );
        return NULL;
    }

    vector<CCRingNode*>::iterator i;
    for ( i = m_rt.begin(); i != m_rt.end(); ++i ) {
        if ( *i == last ) {
            ++i;
            break;
        }
    }

    return i != m_rt.end() ? ( *i ) : NULL;
}

void CCRingTable::TrimBranches()
{
    // process nodes of degree 0 and 1
    // iteratively remove all degree 0 and 1 nodes 
    bool FINISH( false );
    CCRingNode* rnode( NULL );

    do {
        FINISH = true;
        for ( rnode = BeginNode(); rnode ; ) {
            if ( rnode->GetDegree() == 0 || rnode->GetDegree() == 1 ) {
                // RemoveNode will return pointer of next node
                FINISH = false;
                rnode = RemoveNode( rnode );
            }
            else {
                rnode = NextNode( rnode );
            }
        }
    }
    while ( !FINISH );
}

CCRingNode* CCRingTable::ProcessNodeTwo()
{
    // process nodes of degree 2
    // iteratively remove chain of N2 nodes
    // if not exists, remove an isolated N2 node
    // return a node with minimum degree
    CCMol* mol = GetMol();
    set<CCRingNode*>::iterator j;
    int minDRG = 77; // it's my birthday :-)
    bool foundChainN2 = false, foundIsolateN2 = false;
    CCRingNode* rnode, * head, * nbrnode, * minDRGNode = NULL,
                * isolateN2 = NULL;

    do {
        minDRG = 77; // my birthday again ;-)
        foundChainN2 = false;
        foundIsolateN2 = false;
        minDRGNode = NULL;
        isolateN2 = NULL;
        set<CCRingNode*> chainN2;

        for ( rnode = BeginNode(); rnode ; ) {
            if ( rnode->GetDegree() < minDRG ) {
                minDRG = rnode->GetDegree();
                minDRGNode = rnode;
            }

            if ( rnode->GetDegree() == 2
              && chainN2.find( rnode ) == chainN2.end() ) {
                chainN2.clear();
                queue<CCRingNode*> q;
                CCBitVec used( mol->GetNumAtom() + 1 );
                q.push( rnode );
                used.Set( rnode->GetIdx() );

                // search for a chain of N2
                while ( !q.empty() ) {
                    head = q.front();
                    q.pop();
                    chainN2.insert( head );

                    for ( nbrnode = head->BeginNbr();
                          nbrnode;
                          nbrnode = head->NextNbr( nbrnode ) ) {
                        if ( !used[nbrnode->GetIdx()] &&   
                            // avoid duplicates in queue
                            nbrnode->GetDegree() == 2 ) {
                            foundChainN2 = true;
                            q.push( nbrnode );
                            used.Set( nbrnode->GetIdx() );
                        }
                    }
                }

                // process the found chain
                if ( foundChainN2 ) {
                    // record SSSR
                    CCRing sssr;

                    // in real program not all degree 2 nodes are necessary
                    // one node represents for its chain
                    if ( FindSSSRing( rnode, sssr ) ) {
                        AddUniqueSSSR( sssr );

                        // break out for next iteration
                        break;
                    }
                    else {
                        // failed to find a sssr, search for next chainN2
                        rnode = NextNode( rnode );
                    }
                }
                else {
                    // an isolate N2 node
                    foundIsolateN2 = true;
                    isolateN2 = rnode;
                    rnode = NextNode( rnode );
                }
            }
            else {
                rnode = NextNode( rnode );
            }
        }

        // trim chainN2
        // a) found a sssr, remove chainN2
        // b) unable to find a sssr, remove the last chainN2
        if ( foundChainN2 && !chainN2.empty() ) {
            for ( j = chainN2.begin(); j != chainN2.end(); ++j ) {
                RemoveNode( *j );
            }
        }

        // only found isolated N2 nodes
        if ( !foundChainN2 && foundIsolateN2 ) {
            // record SSSR
            CCRing sssr;

            if ( FindSSSRing( isolateN2, sssr ) ) {
                AddUniqueSSSR( sssr );
            }

            // remove this node
            RemoveNode( isolateN2 );
        }

        // before next iteration, let's clean branches if any
        TrimBranches();
    }
    while ( foundChainN2 || foundIsolateN2 ); // may still have N2 node

    return minDRGNode;
}

void CCRingTable::BreakEdge( CCRing* sssr )
{
    int smallest = 77; // it's my birthday :-)
    CCBitVec bvpath = sssr->GetBVPath();
    CCRing tmpRing;
    int rsize1, rsize2, rlarger;
    CCRingNode* bgn, * end;
    pair<CCRingNode*, CCRingNode*> bkEdge;

    vector<int> vr = bvpath.ToTureIdx();

    for ( int i = 0; i < vr.size(); ++i ) {
        CCBond* bond = GetMol()->GetBond( vr[i] );
        bgn = GetNode( bond->GetBeginAtom() );
        end = GetNode( bond->GetEndAtom() );
        // break this edge;
        bgn->RemoveNbr( end );
        end->RemoveNbr( bgn );
        // find the sssr for resulting nodes
        rsize1 = FindSSSRing( bgn, tmpRing );
        rsize2 = FindSSSRing( end, tmpRing );
        rlarger = ( rsize1 > rsize2 ) ? rsize1 : rsize2;
        // restore edge
        bgn->AddNbr( end );
        end->AddNbr( bgn );

        if ( rlarger < smallest ) {
            smallest = rlarger;
            bkEdge.first = bgn;
            bkEdge.second = end;
        }
    } // end trial

    // break the bkEdge
    if ( bkEdge.first && bkEdge.second ) {
        bkEdge.first->RemoveNbr( bkEdge.second );
        bkEdge.second->RemoveNbr( bkEdge.first );
    }
    else {
        gErrorLog.Throw( __FUNCTION__, "Something must be wrong!", ccFatal );
    }
}

int CCRingTable::FindSSSRing( CCRingNode* rootnode, CCRing& sssr )
{
    CCMol* mol = GetMol();
    int numatom = mol->GetNumAtom();

    vector<CCBitVec> avPaths( numatom + 1 ); // equals to number of atoms +1
    for ( int k = 1; k < avPaths.size(); ++k ) {
        // avPaths[0] was used to intersection and union
        avPaths[k].Resize( numatom + 1 );
    }           // plays role as a token
    vector<vector<int> > aPaths( numatom + 1 );       // sequential path 

    int ringsize = 0;
    CCBitVec avpath( numatom + 1 ), visit( numatom + 1 ), used( numatom + 1 );
    vector<int> prev( numatom + 1 );
    CCRingNode* head, * nbrnode;
    int rootIdx = rootnode-> GetIdx(), headIdx, nbrIdx;

    // initial queue
    queue<CCRingNode*> q;
    q.push( rootnode );
    used.Set( rootIdx );
    aPaths[rootIdx].push_back( rootIdx );
    avPaths[rootIdx].Set( rootIdx );

    while ( !q.empty() ) {
        head = q.front();
        q.pop();
        headIdx = head->GetIdx();
        visit.Set( headIdx );

        for ( nbrnode = head->BeginNbr();
              nbrnode;
              nbrnode = head->NextNbr( nbrnode ) ) {
            nbrIdx = nbrnode->GetIdx();
            if ( nbrIdx == prev[headIdx] ) {
                continue;
            }
            else if ( !visit[nbrIdx] ) {
                if ( !used[nbrIdx] ) // avoid duplicate push into queue
                {
                    q.push( nbrnode );
                    used.Set( nbrIdx );
                    prev[nbrIdx] = headIdx;
                    aPaths[nbrIdx] = aPaths[headIdx];
                    aPaths[nbrIdx].push_back( nbrIdx );
                    avPaths[nbrIdx].Set( nbrIdx );
                    avPaths[nbrIdx] |= avPaths[headIdx];
                }
            }
            else // collision detected
            {
                avPaths[0].Reset(); // clear
                avPaths[0] = avPaths[headIdx] & avPaths[nbrIdx];
                if ( avPaths[0].Count() == 1 ) // a sssr was found including rootnode (singleton)
                {
                    avpath = avPaths[headIdx] | avPaths[nbrIdx];
                    // get the sequential atom path of sssr
                    vector<int> sssrAtomPath;
                    copy( aPaths[headIdx].begin(), aPaths[headIdx].end(),
                          back_inserter( sssrAtomPath ) );
                    copy( aPaths[nbrIdx].rbegin(), aPaths[nbrIdx].rend(),
                          back_inserter( sssrAtomPath ) );
                    sssrAtomPath.erase( sssrAtomPath.end() - 1 ); // remove last

                    // get ring info.
                    // get the bvpath
                    ringsize = sssrAtomPath.size();
                    CCBitVec bvpath( mol->GetNumBond() + 1 );
                    for ( int i = 0; i < ringsize; ++i ) {
                        bvpath.Set( mol->GetBond( sssrAtomPath[i],
                                                  sssrAtomPath[( i + 1 ) % ringsize] )->GetIdx() );
                        //vatoms[i] = mol->GetAtom( sssrAtomPath[i] );
                    }
                    // set ring data
                    sssr.SetMol( mol );
                    sssr.SetAtoms( sssrAtomPath );
                    sssr.SetBVPath( bvpath );

                    return ringsize;
                }
            }
        }
    }

    return ringsize;
}

CCRingNode* CCRingTable::RemoveNode( CCRingNode* rnode )
{
    if ( !rnode ) {
        gErrorLog.Throw( __FUNCTION__, "Unable to trim a NULL node!",
                         ccWarning );
        return NULL;
    }

    vector<CCRingNode*>::iterator i;
    CCRingNode* nbrNode;

    for ( i = m_rt.begin(); i != m_rt.end(); ) {
        if ( ( *i ) == rnode ) {
            // update connection table for neighbors of rnode
            // remove neighbors of rnode
            for ( nbrNode = rnode->BeginNbr();
                  nbrNode;
                  nbrNode = rnode->NextNbr( nbrNode ) ) {
                nbrNode->RemoveNbr( rnode );
            }

            // erase rnode from m_rt
            i = m_rt.erase( i );

            // release memory
            delete rnode;

            break;
        }
        else {
            ++i;
        }
    }

    return i != m_rt.end() ? ( *i ) : NULL;
}

void CCRingTable::AddUniqueSSSR( const CCRing& sssr )
{
    for ( CCRingIter i = m_vring.begin(); i != m_vring.end(); ) {
        if ( ( *i )->GetBVPath() == sssr.GetBVPath() ) // identical
        {
            return;
        }
        // sssr is (linear to) enclosed in (*i)
        // erase (*i)
        // shall never happpen in BFS algorithm
        else {
            ++i;
        }
    }

    CCRing* sr = new CCRing( sssr ); 
    AssertMemAlloc( sr );
    m_vring.push_back( sr );
}

void CCRingTable::BFSFindSSSR()
{
    do {
        // process nodes of degree 0 and 1
        // iteratively remove all degree 0 and 1 nodes 
        // it seems that there is no need to trim branches because the way of BuildRingTable()
        //rt->TrimBranches(); 
        //if (rt.Empty()) 
        //  break; // rtable is empty

        // process nodes of degree 2
        // iteratively remove chain of N2 nodes
        // if not exists, remove all isolated N2 nodes
        // return a node with minimum degree
        CCRingNode* minDRGNode = ProcessNodeTwo();
        if ( !minDRGNode ) {
            break;
        } // rt is empty

        // process nodes with degree 3
        // select a optimum edge to break
        // result in two N2 nodes for next cycle
        if ( ( minDRGNode->GetDegree() >= 3 ) ) {
            // record SSSRs
            CCRing sssr;
            // return size of sssr
            if ( FindSSSRing( minDRGNode, sssr ) ) {
                AddUniqueSSSR( sssr );
            }

            // break one optimum edge
            BreakEdge( &sssr );
        }
        else {
            continue;
        }
    }
    while ( !Empty() );
}

